-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

-- Const: constants
--
-- available under RPTAGS.const
--
local RPTAGS = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function
  
  local GetAddOnMetadata = GetAddOnMetadata;
  
    -- ----------------- - Temporary values that are used later in rptags.globals
  local CFF               = "|cff";
  local ORA_COLOR         = "bb00bb";
  local VIOLET            = "dd33aa";
  local LIGHT_ORANGE      = "dd9933";
  local RESET             = "|r";
  
  local core = { 
    -- ------------- - addon details
    APP_ID           = "rptags",
    APP_COLOR        = CFF .. VIOLET,
    APP_RPUF_COLOR   = CFF .. LIGHT_ORANGE,
    ORAIBI           = CFF .. ORA_COLOR .. "Oraibi" .. RESET,
    VERSION          = GetAddOnMetadata("RP_Tags", "Version"),
    -- ------------- - Localization prefixes
    PREFIX           = "RPT_",
    LOC              = "",
    FLOC             = "FMT_",
    TAG_PREFIX       = "TAG_",
    BROADCAST_PREFIX = "RPB1",
    -- ------------- - Configuration Options
    VERYSHORT        = 10,
    SHORT            = 20,
    MEDIUM           = 40,
    LONG             = 60,
    VERYLONG         = 150,
    -- ------------- -
    INDENT           = "     ",
    -- ------------- -
    BIG_PROFILE      = 10, 
    -- ------------- - Building tags with these
    TAG              = "rp:",
    MAIN_EVENT       = "PLAYER_TARGET_CHANGED PLAYER_FOCUS_CHANGED",
    -- ------------- - relcolor
    RELCOLOR = { NONE  = "", FRIEND = "|cff00ff00", UNFRIENDLY = "|cffff0000", NEUTRAL = "|cff8080ff", BUSINESS = "|cffffff00", LOVE = "|cffff80ff", FAMILY = "|cffffaa00", },
    -- ------------- - rp:UnitFrames
    RPUF = { INITIAL_POSITION = {
               player       = { pt = 'TOPRIGHT',    relto = 'uiparent',    relpt = 'CENTER',   x = -100, y = -200 },
               focus        = { pt = 'BOTTOMLEFT',  relto = 'RPUF_Player', relpt = 'TOPLEFT',  x =    0, y =    4 },
               target       = { pt = 'TOPLEFT',     relto = 'uiparent',    relpt = 'CENTER',   x =  100, y = -200 },
               targettarget = { pt = 'BOTTOMRIGHT', relto = 'RPUF_Target', relpt = 'TOPRIGHT', x =    0, y =    4 },
               }, },
    -- ------------- - Sizebuffs
    SIZEBUFFS = { 
      -- include them here if they have a duration of 5 min or longer
      [8212]   = { buffName = "Giant Growth Potion",  sizeChange = 30,  foodBuff = false, },
      [98444]  = { buffName = "Vrykul Drinking Horn", sizeChange = 30,  foodBuff = false, },
      [58466]  = { buffName = "Giant Feast",          sizeChange = 30,  foodBuff = true,  },
      [58479]  = { buffName = "Small Feast",          sizeChange = -30, foodBuff = true,  },
      [143034] = { buffName = "Darkmoon Seesaw",      sizeChange = -33, foodBuff = false, },
      [17038]  = { buffName = "Winterfall Firewater", sizeChange = 25,  foodBuff = false, },
      [109933] = { buffName = "Darkmoon Firewater",   sizeChange = 25,  foodBuff = false, },
      [16595]  = { buffName = "Noggenfogger Elixir",  sizeChange = -50, foodBuff = false, },
    },
  
  -- new -------------------------------------------------------------------------------------------------------
    SERVER = { -- only RP servers
      ID = { [""] = 0,
             ["Silver Hand"] = 12,
             ["Argent Dawn"] = 75,
             ["Cenarion Circle"] = 88,
             ["Earthen Ring"] = 100,
             ["Feathermoon"] = 118,
             ["Shadow Council"] = 125,
             ["Scarlet Crusade"] = 126,
             ["Emerald Dream"] = 162,
             ["Maelstrom"] = 163,
             ["Twisting Nether"] = 164,
             ["Forscherliga"] = 516,
             ["Argent Dawn"] = 536,
             ["Kirin Tor"] = 537,
             ["Earthen Ring"] = 561,
             ["Die Silberne Hand"] = 576,
             ["Zirkel des Cenarius"] = 592,
             ["Kult der Verdammten"] = 613,
             ["Das Syndikat"] = 614,
             ["Der Rat von Dalaran"] = 617,
             ["Defias Brotherhood"] = 635,
             ["The Venture Co"] = 636,
             ["Conseil des Ombres"] = 644,
             ["Les Sentinelles"] = 647,
             ["Kirin Tor"] = 1071,
             ["Moonglade"] = 1085,
             ["La Croisade écarlate"] = 1086,
             ["Scarshield Legion"] = 1096,
             ["Steamwheedle Cartel"] = 1117,
             ["Die ewige Wacht"] = 1118,
             ["Die Todeskrallen"] = 1119,
             ["Die Arguswacht"] = 1121,
             ["Confrérie du Thorium"] = 1127,
             ["Lightninghoof"] = 1130,
             ["Thorium Brotherhood"] = 1154,
             ["Steamwheedle Cartel"] = 1260,
             ["The Venture Co"] = 1289,
             ["Sentinels"] = 1290,
             ["Ravenholdt"] = 1308,
             ["Darkmoon Faire"] = 1317,
             ["Der Abyssische Rat"] = 1326,
             ["Der Mithrilorden"] = 1327,
             ["Die Nachtwache"] = 1333,
             ["Culte de la Rive noire"] = 1337,
             ["Blackwater Raiders"] = 1347,
             ["Ravenholdt"] = 1352,
             ["Sisters of Elune"] = 1356,
             ["Moon Guard"] = 1365,
             ["Wyrmrest Accord"] = 1369,
             ["Farstriders"] = 1370,
             ["Todeswache"] = 1405,
             ["The Scryers"] = 1570,
             ["The Sha'tar"] = 1595,
             ["Sporeggar"] = 1606,
             ["Die Aldor"] = 1618,
             ["Das Konsortium"] = 1619,
             ["Les Clairvoyants"] = 1626,
           }, -- ID
  
      TYPE = {
             [0]="PVE",
             [12]="RP",
             [75]="RP",
             [88]="RP",
             [100]="RP",
             [118]="RP",
             [125]="RP",
             [126]="RP",
             [162]="RP",
             [163]="RP",
             [164]="RP",
             [516]="RP",
             [536]="RP",
             [537]="RP",
             [561]="RP",
             [576]="RP",
             [592]="RP",
             [613]="RP",
             [614]="RP",
             [617]="RP",
             [635]="RP",
             [636]="RP",
             [644]="RP",
             [647]="RP",
             [1071]="RP",
             [1085]="RP",
             [1086]="RP",
             [1096]="RP",
             [1117]="RP",
             [1118]="RP",
             [1119]="RP",
             [1121]="RP",
             [1127]="RP",
             [1130]="RP",
             [1154]="RP",
             [1260]="RP",
             [1289]="RP",
             [1290]="RP",
             [1308]="RP",
             [1317]="RP",
             [1326]="RP",
             [1327]="RP",
             [1333]="RP",
             [1337]="RP",
             [1347]="RP",
             [1352]="RP",
             [1356]="RP",
             [1365]="RP",
             [1369]="RP",
             [1370]="RP",
             [1405]="RP",
             [1570]="RP",
             [1595]="RP",
             [1606]="RP",
             [1618]="RP",
             [1619]="RP",
             [1626]="RP",
           }, -- TYPE
  
      ABBR = { -- only enUS realms
            [0]="",
            [12]="SH",
            [75]="AD",
            [88]="CC",
            [100]="ER",
            [118]="FM",
            [125]="ShC",
            [126]="ScC",
            [162]="ED",
            [163]="M",
            [164]="TN",
            [536]="AD",
            [537]="KT",
            [561]="ER",
            [635]="DB",
            [636]="VC",
            [1071]="KT",
            [1085]="MG",
            [1096]="SL",
            [1117]="SC",
            [1130]="LH",
            [1154]="TB",
            [1260]="StC",
            [1289]="VC",
            [1290]="S",
            [1308]="RH",
            [1317]="DMF",
            [1347]="BR",
            [1352]="RH",
            [1356]="SoE",
            [1365]="MG",
            [1369]="WRA",
            [1370]="FS",
            [1570]="Scr",
            [1595]="Sha",
            [1606]="Spo",
          }, -- ABBR
      }, -- server
    CLIENT = {
      LOOKUP = {
        ["Total RP 3"]     = "TRP",
        ["MyRolePlay"]     = "MRP",
        ["XRP"]            = "XRP",
        ["xrp"]            = "XRP",
        ["TRP3: Extended"] = "TRPE",
        }, -- lookup
      ICON = { 
        TRP = "",
        MRP = "",
        XRP = "",
        TRPE = "",
      },
     }, -- client
  
    VISIBILITY = {  -- not yet implemented
      -- methodology:
        --
        -- Window with x checkboxes
        --
        -- When checked, create a concatenated list ";" with "show" at the end
        --
        -- Save this to the frame and update
                                                      -- defaults
      HIDE_IN_COMBAT          = "[combat]      hide", -- true
      HIDE_NOT_RESTING        = "[noresting]   hide", -- false
      HIDE_IN_PARTY           = "[group:party] hide", -- false
      HIDE_IN_RAID            = "[group:raid]  hide", -- true
      HIDE_WHEN_DEAD          = "[dead]        hide", -- true
      HIDE_WHEN_SHAPESHIFTED  = "[shapeshift]  hide", -- false
      HIDE_IN_STEALTH         = "[stealth]     hide", -- false
      HIDE_IN_VEHICLE         = "[vehicleui]   hide", -- true
      HIDE_IN_PET_BATTLE      = "[petbattle]   hide", -- true
      }, -- visibility
    BACKDROP = {
      BLIZZTOOLTIP = { -- bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background",
                       bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                       edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
                       tileSize = 16,
                       edgeSize = 16,
                       insets   = { left = 5, right = 5, top = 5, bottom = 4} },
      ORIGINAL     = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                       insets   = {top = -1, bottom = -1, left = -1, right = -1} },
      }, -- backdrop
    ANIM_TIMER = 6, -- seconds
    ANIM = { -- animations              -- camera, anim, cycle
--       BOW        = { 1,  66, false },
--       CHEER      = { 1,  68, false },
--       DANCE      = { 1,  69,  true },
--       EXCITED    = { 1,  64, false },
      FACE       = { 1,   0,  true },
--       FLEX       = { 0,  83, false },
--       FLIRT      = { 1,  83, false },
      FULLBODY   = { 0,   0,  true },
--       KISS       = { 0,  76, false },
--       LAUGH      = { 0,  70, false },
--       READY      = { 1,  26,  true },
--       SALUTE     = { 0, 113, false },
--       SHOUT      = { 1,  81, false },
--       SHRUG      = { 0,  65, false },
--       SIT        = { 1,  97,  true },
--       TALK       = { 0,  60, false },
--       WAVE       = { 0,  67, false },
       }, -- animations

  }; -- core

  for k, v in pairs(core) do RPTAGS.const[k]=v end;

  return "const-core";
end);  -- startup function
  

