-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Help: the /rptags command

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function

  local Const      = RPTAGS.const;
  local Utils      = RPTAGS.utils;
  local Cache      = RPTAGS.cache;
  
  local loc        = Utils.locale.loc;
  local hilite     = Utils.text.hilite;
  local Config     = Utils.config;
  
  local TAG_PREFIX = Const.TAG;
  
  if not Cache.helpdata then Cache.helpdata = {} end;
  local Help = Cache.helpdata;

  -- create the reference menu ------------------------------------------------------------------------
  Utils.config.reference(TRP3_ConfigurationWidgetrptags_reference_topic);
  
        -- formats and sends a help message, used by the /rptags command -------------------------
  local function helpCmd(topic)
  
        local helpFunctions = {
           ["HELP_PARAM_BLANK"]
             = function()
                 return loc("HELPTOPIC_DEFAULT")
                 end, -- function
           ["HELP_PARAM_LIST"] 
             = function()
                 local h = loc("HELP_LIST_START");
                 for k, _ in pairs(Help)
                   do if string.find(string.lower(k), TAG_PREFIX) == 1
                         then h = h .. "[" .. k .. "]  ";
                         end; -- i f
                   end; -- for
                 h = h .. loc("HELP_LIST_END");
                 return h
                 end, -- function
           ["HELP_PARAM_VERSION"]
             = function()
                 local GetBuildInfo = GetBuildInfo;
                 local GetAddOnMetadata = GetAddOnMetadata;
                 local wowBuild, _, _, _ = GetBuildInfo();
         
                 if RPTAGS.mode == "elvui"
                 then return string.format("Version %s by %s ([[ElvUI Mode]], [[[WoW %s]]], [[TRP3 %s]], [[[ElvUI %s]]])", 
                             Const.VERSION, Const.ORAIBI, wowBuild, TRP3_API.globals.version_display, 
                              GetAddOnMetadata("ElvUI", "Version"));
                 elseif RPTAGS.mode == "standalone"
                 then return string.format("Version %s by %s ([[Standalone Mode]], [[[WoW %s]]], [[TRP3 %s]])", 
                             Const.VERSION, Const.ORAIBI, wowBuild, TRP3_API.globals.version_display);
                 elseif RPTAGS.mode == "parallel"
                 then return string.format("Version %s by %s ([[Parallel Mode]], [[[WoW %s]]], [[TRP3 %s]], [[[ElvUI %s]]])", 
                             Const.VERSION, Const.ORAIBI, wowBuild, TRP3_API.globals.version_display, 
                              GetAddOnMetadata("ElvUI", "Version"));
                 end; -- if
                 end,-- function
           ["HELP_PARAM_CHANGES"]
             = function()
                 return "[[" .. loc("CHANGES") .. "]]: " .. loc("APP_CHANGES");
                 end, -- function
           ["HELP_PARAM_DUMP_SETTINGS"]
             = function()
                 local h = "";
                 for k, v in pairs(Const.CONFIG.KEYS)
                   do h = h .. "\n[" .. v .. '] = "[[' .. tostring(Utils.config.get(k)) .. ']]"';
                   end; -- for
                 return h;
                 end, -- function
          } -- help functions
  
        local h;
  
        if not topic then topic = "" end;
        local found = false;
        topic = string.lower(topic);
        for k, f in pairs(helpFunctions)
            do if topic == loc(k) then h = f(topic); found = true end;
            end; -- for
        if not found 
           then if     Help[topic]               then h = loc("HELPTOPIC_" .. Help[topic])
                elseif Help[TAG_PREFIX .. topic] then h = loc("HELPTOPIC_" .. Help[TAG_PREFIX .. topic])
                else   h = loc("HELP_ERROR")
                end; -- if/elseif/else
           end; -- if not found
  
        print(
          string.format(
            loc("FMT_HELP"), 
            loc("APP_COLOR_NAME"), 
            hilite(h) 
          ) -- string.format
        ); -- print
  
    end; -- function
  
  -- set up the slash commands
  SLASH_RPTAGS1          = loc("APP_SLASH");
  SLASH_RPTAGS2          = loc("APP_SLASH_SHORT");
  SLASH_RPTAGS3          = loc("APP_SLASH_SHORTER");
  
  SlashCmdList["RPTAGS"] = function(a) helpCmd(a); end;
  
  -- if the user wants the login message, print it
  if   Config.get("LOGIN_MESSAGE")
  then print(
         hilite(
           string.format(
             loc("FMT_APP_LOAD"), 
             loc("APP_NAME"), 
             loc("APP_VERSION"), 
             loc("MODE_" .. RPTAGS.mode:upper()), 
             loc("APP_SLASH")
           ) -- string.format
         ) -- hilite
       ); -- print
  end; -- if
  
  if   Config.get("CHANGES_MESSAGE")
  then print(
         hilite(
           loc("APP_COLOR_NAME").." v"..loc("APP_VERSION")
               .." [["..loc("CHANGES").."]]: "..loc("APP_CHANGES")
         ) -- hilite
       ); -- print
  end; -- if
  
  return "help";
end); -- startup function  
