-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Get-Icons: functions to return a ui escape sequence with a texture
  --
  -- function list at the end

local RPTAGS           = RPTAGS;
table.insert(RPTAGS.startup, function() -- startup function 
  local Const            = RPTAGS.const;
  local Utils            = RPTAGS.utils;
  local Get              = RPTAGS.get;
  
  if not Get.icon then Get.icon = {} end;
  local Icon = Get.icon;
  
  local loc              = Utils.locale.loc;
  local Config           = Utils.config.get;
  local splitRGB         = Utils.color.hexaToNumber;
  local match            = Utils.text.match;
  
  local glance           = Get.text.glance;
  local field            = Get.core.field;
  local chara            = Get.core.chara;
  local gender           = Get.text.gender;
  local relation         = Utils.import.trp3.relation;
  local getProfileID     = Get.core.profileID
  
  local ICON             = Const.icons;
  local STATUS           = ICON.STATUS;
  local GENDER           = ICON.GENDER; -- category
  local PARAMS           = ICON.PARAMS;  -- category
  
  local function ico(path, tint, param)
        if not path  then return ""     end;
  
        if        not param            then param = PARAMS.DEFAULT
           elseif PARAMS[param] ~= nil then param = PARAMS[param]
           else                             param = PARAMS.DEFAULT
           end; -- if not param
  
        local val;
        val = "|TInterface\\";
        val = val .. path;
        val = val .. ":" .. param;
  
        if tint then local r, g, b = splitRGB(tint);
                     val = val .. ":" .. r .. ":" .. g .. ":" .. b;
                end; -- if tint
        val = val .. "|t";
        return val;
  end; 
  
  local function getGlanceIcons(u)
        if UnitIsPlayer(u)
           then return glance(1, u, false, false, true)
                   ..  glance(2, u, false, false, true)
                   ..  glance(3, u, false, false, true)
                   ..  glance(4, u, false, false, true)
                   ..  glance(5, u, false, false, true);
           else return "";
        end; -- if
  end; 
  
        -- returns the default user icon
  local function unitIcon(u)
        local iconName = chara("IC", u, "UNLIM");
        if not iconName or iconName == "" then return ""; end; -- print nothing if they have no icon?
        return ico("ICONS\\" .. iconName);
  end; 
  
  local function getGenderIcon(u)
        local genderField, genderCode = gender(u);
        if not match(genderField, loc("GENDER_UNKNOWN")) and genderCode == "UNKNOWN"
           then return unitIcon(u); -- if they have a unique gender field then return their picture
           else return ico(GENDER[genderCode], Config("COLOR_" .. genderCode), genderCode)
           end; -- if/then/else
  end; 
  
  local function getStatusIcon(u)
        local tint;
        if UnitIsPlayer(u)
           then local status = field("character/RP", u); -- get the status
                if        status == "1" then tint = Config("COLOR_IC")
                   elseif status == "2" then tint = Config("COLOR_OOC");
                   else                      tint = Config("COLOR_UNKNOWN")
                   end; -- if/elseif/else
           else tint = Config("COLOR_NPC");
           end; -- if player
        return ico(STATUS, tint);
  end;  
  
  local function relationshipIcon(u)
        local profileID = getProfileID(u);
        if profileID then return ico("ICONS\\" .. relation.texture(profileID)) else return ""; end; -- if
  end; 
  
  local function pvpIcon(u)
        if UnitIsPlayer(u) and UnitIsPVP(u) then return ico("PVPFrame\\Icon-Combat") else return "" end;
  end;
        
  -- functions available under RPTAGS.get.icon
  --
  Icon.gender  = getGenderIcon;
  Icon.glances = getGlanceIcons;
  Icon.status  = getStatusIcon;
  Icon.unit    = unitIcon;
  Icon.relation = relationshipIcon;
  Icon.pvp     = pvpIcon;

  return "get-icons";
end);

