-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Get

local RPTAGS           = RPTAGS;
table.insert(RPTAGS.startup, function() 
  local Const            = RPTAGS.const;
  local Utils            = RPTAGS.utils;
  local Get              = RPTAGS.get;
  local LRI              = LibStub("LibRealmInfo");
  
  if not Get.text then Get.text = {}; end;
  local Text = Get.text;
  
  local Config           = Utils.config.get;
  local fmt              = string.format;
  local loc              = Utils.locale.loc;
  local floc             = Utils.locale.floc;
  local Format           = Utils.format; -- the subcategory
  local Parse            = Utils.parse; -- the subcategory
  local wrap             = Utils.text.wrap;
  local split            = Utils.text.split;
  local tc               = Utils.text.titlecase;
  local lc               = string.lower;
  local uc               = string.upper;
  local relation         = Utils.import.trp3.relation;
  
  local getTrpName       = Get.core.trpName;
  local field            = Get.core.field;
  local chara            = Get.core.chara;
  local misc             = Get.core.misc;
  local getUnitID        = Get.core.unitID;
  local getProfile       = Get.core.profile;
  local getData          = Get.core.data;
  local getProfileID     = Get.core.profileID;
  local getBaseData      = Get.core.baseData;
  
        -- functions that just get and return straight values
  local function getAge(u, w)      
        return chara("AG", u, w and "MISC" or "UNLIM"); 
        end; 
  
  local function getBP(u, w)
        return chara("BP", u, w and "MISC" or "UNLIM"); 
        end; 
  
  local function getCurr(u, w) 
        return field("character/CU", u, w and "CURR" or "UNLIM"); 
        end; 
  
  local function getEyes(u, w) 
        return chara("EC", u, w and "MISC" or "UNLIM"); 
        end; 
  
  local function getFT(u, w) 
        return chara("FT", u, w and "TITLE" or "UNLIM"); 
        end; 
  
  local function getHome(u, w) 
        return chara("RE", u, w and "MISC" or "UNLIM");
        end; 
  
  local function getInfo(u) 
        return field("character/CO", u, w and "CURR" or "UNLIM"); 
        end; 
  
  local function getLastName(u) 
        return chara("LN", u, w and "HALFNAME" or "UNLIM");
        end; 
  
  local function getTitle(u) 
        return chara("TI", u, w and "HALFTITLE" or "UNLIM");
        end; 
  
        -- retrieve at-first-glance fields ---------------------------------------------------------------------------
  local function getGlance(u, slot, showTitle, showText, showIcon, nowrap)
        if not UnitIsPlayer(u) then return ""; end; -- if
        showTitle = showTitle == 1;
        showText = showText == 1;
        showIcon = showIcon == 1;
  
        local delimiter = Config("GLANCE_DELIM");
        local separator = Config("GLANCE_COLON");
  
        if slot == 0
           then local  allGlances = {};
                for i =  1, 5 
                  do local iGlance =  getGlance(u, i, showTitle, showText, showIcon, true);
                     if iGlance ~= "" then table.insert(allGlances, iGlance) end;
                  end -- for
                return wrap(table.concat(allGlances, delimiter or ""), showTitle and "GLANCETITLE" or "GLANCE");
           else local  path    = "misc/PE/" .. slot;
                local  unitID  = getUnitID(u);
                local  profile = getProfile(unitID);
  
                local  active = getData(path .. "/AC", profile);
                if not active then return "" end;
  
                local title = field(path .. "/TI", u, nowrap and "UNLIM" or "MISC");
                local text =  field(path .. "/TX", u, nowrap and "UNLIM" or "GLANCE");
                local icon =  field(path .. "/IC", u, "UNLIM");
                local value = "";
  
                if   showIcon and icon
                then value = value .. "|TInterface\\ICONS\\" .. icon .. ":0|t "; 
                end;

                if   showTitle and title
                then value = value .. title;                                     
                end;

                if   showTitle and showText and text and title
                then value = value .. separator;                                 
                end;

                if   showText and text
                then value = value .. text;                                      
                end;

                if   value == "" and showTitle 
                then value = "..."; 
                end;

                return value;
  
           end  -- if
  end   
  
        -- whether the unit has any buffs affecting their scale -----------------------------------------------
  local function getSizeBuff(unit)
        local sizeBuff = nil;
        if UnitIsPlayer(unit)
           then for buffID,_ in pairs(Const.SIZEBUFFS)
                  do for i = 1,40
                     do local _, _, _, _, _, _, _, _, _, spellID = UnitAura(unit,i);
                        if spellID == buffID then sizeBuff = buffID; end;
                     end; -- for i = 1,40
                  end; -- for buffID
                if sizeBuff -- at this point either we found a buff or we haven't
                   then return Format.sizebuff(sizeBuff, 
                                   Config("SIZEBUFF_FMT"), 
                                   UnitRace(unit) == "PANDAREN"); 
                   else return "";
                   end; -- if sizeBuff
           else return "";
           end; -- if playser
  end   
  
        -- get the height of a player --------------------------------------------------------------------------------
  local function getHeight(u) 
        if not UnitIsPlayer(u) then return ""; end; -- if
  
        local height = chara("HE", u, "UNLIM"); 
        if    height == "" then return ""; end; -- if
  
        local cm, s;
        if Config("PARSE_HW") 
           then cm, s = Parse.height(height)  -- we don't care about the exactness
           end; -- if
  
        if s -- if we parsed it and got a cm result
           then return Format.cm(cm,     Config("UNITS_HEIGHT"));
           else return Format.cm(height, Config("UNITS_HEIGHT"));
           end; -- if
  end   
  
        -- get the weight of a player --------------------------------------------------------------------------------
  local function getWeight(u, noParse) 
        if not UnitIsPlayer(u) then return ""; end; -- if
  
        local weight = chara("WE", u, noParse and "MISC" or "UNLIM"); 
        if    weight == "" then return ""; end; -- if
  
        local kg, s;
        if Config("PARSE_HW") and not noParse
           then kg, s = Parse.weight(weight)
           end; --  if
           
        if s -- we parsed it and got a kg result
           then return Format.kg(kg,     Config("UNITS_WEIGHT")); 
           else return Format.kg(weight, Config("UNITS_WEIGHT")); 
           end; -- if
  
  end   
  
        -- get the race of a player or other unit --------------------------------------------------------------------
  local function getRace(u, w)
        local trpVersion = chara("RA", u, w and "MISC" or "UNLIM");
        local systemRace = wrap(UnitRace(u), w and "MISC" or "UNLIM");
        if string.len(trpVersion) > 0 
           then return trpVersion 
           else return systemRace
           end  -- if trpversion
  end   
  
        -- get the class of a player or another unit -----------------------------------------------------------------
  local function getClass(u, w)
        local trpVersion     = chara("CL", u, w and "MISC" or "UNLIM");
        local systemClass, _ = Utils.text.titlecase(UnitClassBase(u));
        systemClass = wrap(systemClass, w and "MISC" or "UNLIM");
        if string.len(trpVersion) > 0
           then return trpVersion
           else return systemClass
           end  -- if player
  end   
  
        -- get the name of a player or other unit ---------------------------
  local function getName(u, w)
  
        local  sysName = UnitName(u);
        local  trpName = getTrpName(u) or "";
  
        if string.len(trpName) > 0
           then return wrap(trpName, w and "NAME" or "UNLIM");
           else return wrap(sysName, w and "NAME" or "UNLIM");
           end; --if 
  end   
  
        -- get the first name of a player or other unit ---------------------------
  local function getFirstName(u, w)
        local firstName;
  
        local sysName      = wrap(UnitName(u),   w and "HALFNAME" or "UNLIM");
        local trpFirstName = chara("FN", u,      w and "HALFNAME" or "UNLIM");
        local trpFullName  = wrap(getTrpName(u), w and "HALFNAME" or "UNLIM");
  
        if        string.len(trpFirstName) > 0 then firstName = trpFirstName;
           elseif string.len(trpFullName)  > 0
                  then local fn = string.match(trpFullName, "^(%S+)");
                       if fn then firstName = fn else firstName = trpFullName end;
                  else local fn = string.match(sysName,     "^(%S+)");
                       if fn then firstName = fn else firstName = sysName;    end;
           end; -- if
  
        return firstName;
  end   
  
        -- years, an age function that returns a number of years
  local function getYears(u, w, flag)
        local age = chara("AG", u, "UNLIM");
  
        if age == "" then return ""; end;
  
        local years, s, x = Parse.age(age);
        age = wrap(age, w and "MISC" or "UNLIM");
  
        if not s then return age; end;
        if not x then years = loc("ABOUT") .. " " .. years end;
        years = years .. " " .. loc("AGE_YEARS");
        if flag then years = years .. " " .. loc("YEARS_" .. string.upper(flag)) end;
        return years;
  end; 
  
  
        -- ic/ooc status
  local function getStatus(u)
        if UnitIsPlayer(u)
           then local status =  field("character/RP", u, "UNLIM"); -- get the status
                if status == "1" 
                  then return loc("IS_IC")
                  else return loc("IS_OOC")
                  end;
  
           else return loc("IS_NPC");
           end; -- if isplayer
  end   
  
  
  local function getNpcStatus(u)
        if not UnitIsPlayer(u) 
           then return loc("IS_NPC") 
           else return "" 
           end;
  end; 
  
  
  local function getIcStatus(u)
        local status = field("character/RP", u, "UNLIM"); -- get the status
        if status == "1" 
           then return loc("IS_IC") 
           else return ""
           end;
  end; 
        
  local function getOocStatus(u)
        local status = field("character/RP", u, "UNLIM"); -- get the status
        if status == "2" 
           then return loc("IS_OOC") 
           else return ""
           end;
  end; 
        
  local function getRelation(u)
        if not u then return "" end;
        local profileID = getProfileID(u);
        if not profileID then return "" end;
        local rel = relation.get(profileID);
        if not rel then return "" end;
        if rel then return relation.text(profileID) else return "" end;
  end;
  
  local function getGender(u)
        local race        = getRace(u);
        if not race then return loc("GENDER_UNKNOWN"), "UNKNOWN" end;

        local gameSex     = UnitSex(u);
        local genderField = misc("gender", u, "UNLIM");

        if not genderField then return loc("GENDER_UNKNOWN"), "UNKNOWN" end;

        if Config("PARSE_GENDER") 
           then local knownGender, s;
  
                if string.len(genderField) > 0 
                   then knownGender, s = Parse.gender(genderField);
                        if s then return genderField, knownGender 
                             else return genderField, "UNKNOWN"; end;
                   end; -- if genderfield
  
           knownGender, s = Parse.gender(string.gsub(race, "[Hh]uman", "#####"));
           if s then return loc("GENDER_" .. knownGender), knownGender; end;
        end; -- if parse gender
  
        -- so now we'll try what the game thinks they are
        if gameSex == 3 then return loc("GENDER_FEMALE"), "FEMALE"; end;
        if gameSex == 2 then return loc("GENDER_MALE"),   "MALE";   end;
        if gameSex == 1 then return loc("GENDER_NEUTER"), "NEUTER"; end;
  
        -- fallback on "heck i dunno"
        return loc("GENDER_UNKNOWN"), "UNKNOWN";
  
  end; 
  
  local function getPronoun(u, pro)
        -- load the localized pronoun data -------------------------------------------------------------------------------
        local pronounList = loc("PRONOUN_TAGS");
        local genderList  = loc("GENDERS") .. ";" .. loc("EXTRA_PRONOUN_SETS");
  
        -- check the input parameters ------------------------------------------------------------------------------------
        if not pro or not u                       then return "", nil end; -- if we don't have both params
        if not string.len(pro) == 1               then return "", nil end; -- if code for pronoun isn't 1 char
        if not string.match(pronounList, lc(pro)) then return "", nil end; -- if code isn't on the pronoun list
  
        local function case(s)  -- helper function -----------------------------------------------------------------------
              if string.match(pro, "%u") 
                 then return tc(s) 
                 else return lc(s) 
                 end -- if
              end;
  
        local function pronoun(g, v) -- helper function ------------------------------------------------------------------
              return loc("PRONOUN_" .. uc(g) .. "_" .. uc(v));
              end;
  
        local function allPronouns(g)
              local all = "";
              for t in string.gmatch(pronounList, "%a")
                  do all = all .. "/" .. pronoun(g, t) end;
              return all:sub(2);
              end;
              
        local pronounsField = misc("pronouns", u, "UNLIM"); -- get the pronouns field ---------------------------------------------
        local pronounIndex  = string.find(lc(pronounList), lc(pro));
  
        if pronounsField ~= ""  
           then -- they have a custom pronouns field ---------------------------------------------------------------------
  
                local customPronouns = split(pronounsField, "/");
                for _, g in ipairs(split(genderList, ";")) 
                    do -- loop g == loop through the genders -------------------------------------------------------------
                       if lc(pronounsField) == lc(g) -- Does this gender's name match the gender field? ------------------
                          then return case(pronoun(g, pro)), allPronouns(g); end; -- if
  
                       for _, c in ipairs(customPronouns)
                           do -- loop c == loop through their customs ----------------------------------------------------
                              for t in string.gmatch(pronounList, "%a") 
                                  do -- loop t == loop through the tags --------------------------------------------------
                                     if lc(pronoun(g, t)) == lc(c) -- found a match, so we'll assume this is our gender
                                        then return case(pronoun(g, pro)), allPronouns(g); 
                                        end -- if
                                     end -- loop t
                            end -- loop c
  
                    end -- loop g
  
                -- did not find a match on any of the sets but they still have a custom field ----------------------------
                --
                local p = customPronouns[pronounIndex];
                if p then return case(p),                 pronounsField;
                     else return case(customPronouns[1]), pronounsField;
                     end -- if
  
           else -- they don't have a pronouns field ----------------------------------------------------------------------
  
                local _, genderMatch = getGender(u);
                local p = pronoun(genderMatch, lc(pro));
                if p ~= "" then return case(p), allPronouns(genderMatch)
                           else return "",      ""
                           end; -- if 
  
           end; -- if pronounsField --------------------------------------------------------------------------------------
  end;
  
  local function getAllPronouns(u)
        local s = loc("PRONOUN_TAGS"):sub(1,1);
        local _, all = getPronoun(u, s);
        return all;
  end;
  
  local function getMiscAndTweakOutput(miscField, u, w)
    
    local Tweaks = { 
          ["motto"]    = function(unit, w) local  m, f = misc("motto", unit, w and "MISC" or "UNLIM");
                                        return f and fmt(floc("MOTTO"), m) or "" end,
          ["nick-quoted"] = function(unit, w) local n, nf = misc("nick", unit, w and "MISC" or "UNLIM");
                                              local nn, nnf = misc("nickname", unit, w and "MISC" or "UNLIM");
                                              return nf and fmt(floc("MOTTO"), n) or nnf and fmt(floc("MOTTO"), nn) or "" end,
  
          ["house"]    = function(unit, w) local  h, hf = misc("house", unit, w and "MISC" or "UNLIM");
                                        local  f, ff = misc("family", unit, w and "MISC" or "UNLIM");
                                        return hf and h or ff and f or misc("tribe", w and "MISC" or "UNLIM") end,
  
          ["family"]   = function(unit, w) local  f, ff = misc("family", unit, w and "MISC" or "UNLIM");
                                        local  h, hf = misc("house", unit, w and "MISC" or "UNLIM");
                                        return ff and  f or hf and h or misc("tribe", w and "MISC" or "UNLIM") end,
  
          ["tribe"]    = function(unit, w) local  t, tf = misc("tribe", unit, w and "MISC" or "UNLIM");
                                        local  f, ff = misc("family", unit, w and "MISC" or "UNLIM");
                                        return tf and  t or ff and f or misc("house", w and "MISC" or "UNLIM") end,
  
          ["markings"] = function(unit, w) local  m, f =  misc("markings", unit, w and "MISC" or "UNLIM");
                                        return f  and  m or misc("tattoos", unit, w and "MISC" or "UNLIM") end,
  
          ["religion"] = function(unit, w) local  r, f =  misc("religion", unit, w and "MISC" or "UNLIM");
                                        return f  and  r or misc("faith", unit, w and "MISC" or "UNLIM") end,
  
          ["rstatus"]  = function(unit, w) local trp_rs = chara("RS", u, w and "MISC" or "UNLIM");
                                        if trp_rs then return loc("RSTATUS_" .. trp_rs)
                                                  else local  r, rf = misc("relationship status", unit, w and "MISC" or "UNLIM");
                                                       local  m, mf = misc("marital status", unit, w and "MISC" or "UNLIM");
                                                       return rf and  r or mf and m or misc("romantic status", unit, w and "MISC" or "UNLIM") 
                                                  end; end,
       };
  
    if Tweaks[miscField] 
       then return Tweaks[miscField](u, w) 
       else return misc(miscField, u, w and "MISC" or "UNLIM") 
       end; -- if
  end;
  
  -- new -----------------------------------------------------------------------------------------------------------------------------------
  
  local function getNote(u, noteNum)
        return  u and (getUnitID(u) ~= getUnitID('player'))
                  and TRP3_API.profile.getPlayerCurrentProfile().notes
                  and TRP3_Register.character[getUnitID(u)]
                  and TRP3_Register.character[getUnitID(u)].profileID
                  and TRP3_API.profile.getPlayerCurrentProfile().notes[TRP3_Register.character[getUnitID(u)].profileID]
                  or ""
  end; -- function
  
  local function scanNote(u, patNum, iconRequested)
        local thisNote = getNote(u, patNum) .. "\n";
        local matches, found = {}, false;
  
        local pattern = Config("NOTE_" .. patNum .. "_STRING") or "";
        pattern = pattern:lower();
        if pattern:len() >= 3
           then for line in thisNote:gmatch("(.-)\n")
                    do for m in line:lower():gmatch(pattern)
                           do found = true; table.insert(matches, line); end; -- for m
            end; -- for line
        end; -- if
  
        return    found and iconRequested and "|TInterface\\ICONS\\" .. Config("NOTE_" .. patNum .. "_ICON") .. ":0|t "
               or found                   and table.concat(matches, "\n")
               or "";
  end; -- function
        
  local function getKnownName(u)
        local relation = getRelation(u);
        if relation:len() > 0
           then return getName(u)
           else local g, _ = getGender(u);
                local r    = getRace(u);
                return fmt(floc("A"), g .. " " .. r);
           end; -- if
  end; -- function
  
  local function getRelWho(u, w, possessive, prefixName)
        local relation = getRelation(u);
        if relation:len() > 0
           then relation = loc("RELATION_" .. relation:upper())
                if     possessive == "my"   then relation = loc("WORD_MY")   .." ".. relation;
                elseif possessive == "your" then relation = loc("WORD_YOUR") .." ".. relation;
                end;
                if     prefixName then relation = getName(u, w) .. ", " .. relation; end;
           else relation = loc("RELATION_STRANGER")
           end;
        return relation;
  end;
  
  local function getServerInfo(u, request)
    if not UnitIsPlayer(u) then return "" end;
  
    local unitGUID = UnitGUID(u);
    if not unitGUID then return "" end;
  
    if   LRI
    then 
         -- get all the data we need
         local realmID, realmName, apiName, rules, realmLang, _, region, 
               timezone, connected, englishName, englishAPIName = LRI:GetRealmInfoByGUID(unitGUID);
  
         if     request == "name"       then return englishName or realmName
         elseif request == "abbr"       then return Const.SERVER.ABBR[realmID]
         elseif request == "lang"       then return loc("LANG_" .. (realmLang or ""))
         elseif request == "lang-short" then return realmLang
         elseif request == "mine"       then return UnitIsSameServer(u) and (englishName or realmName) or ""
         elseif request == "notmine"    then return UnitIsSameServer(u) and "" or (englishName or realmName)
         elseif request == "region"     then return loc("REGION_" .. region);
         elseif request == "star"       then return UnitIsSameServer(u) and "" or "*"
         elseif request == "dash"       then return UnitIsSameServer(u) and "" or "-"
         elseif request == "subregion"  then return loc("REGION_" .. region .. "_" .. realmLang .. "_" .. (timezone or ""))
         elseif request == "type"       then return loc("SERVER_TYPE_" .. (rules or ""))
         elseif request == "type-short" then return rules
         else                                return ""
         end; -- if request
  
    else local cn, cid, rn, rid, g, n, realmName = GetPlayerInfoByGUID(unitGUID)
         if realmName == "" then realmName = GetRealmName(); end;
         
         if     request == "name"       then return realmName
         elseif request == "abbr"       then return Const.SERVER.ABBR[Const.SERVER.ID[realmName] or 0] or ""
         elseif request == "lang"       then return ""
         elseif request == "lang-short" then return ""
         elseif request == "mine"       then return UnitIsSameServer(u) and (realmName) or ""
         elseif request == "notmine"    then return UnitIsSameServer(u) and "" or (realmName)
         elseif request == "region"     then return ""
         elseif request == "dash"       then return UnitIsSameServer(u) and "" or "-"
         elseif request == "star"       then return UnitIsSameServer(u) and "" or "*"
         elseif request == "subregion"  then return ""
         elseif request == "type"       then return Const.SERVER.TYPE[Const.SERVER.ID[realmName] or 0] or ""
         elseif request == "type-short" then return loc("SERVER_TYPE_" .. Const.SERVER.TYPE[Const.SERVER.ID[realmName] or 0] or "")
         else                                return ""
         end; -- if request
  
     end; -- if LRI
  end; -- function
  
  local function getRoleplayingStyle(u, style, variant)
        local  STYLE = { ic = 1, injury = 2, death = 3, romance = 4, battle = 5, guild = 6, };
        local  num = STYLE[style];
  
        if     num
        then   local value = field("misc/ST/" .. num, u);
               if value == "" then value = 0; end;
               local s = "STYLE_" .. style:upper() .. "_" .. value;
               if variant then s = s .. "_" .. variant; end;
               return loc(s);
        end;  -- if num
  
        local  labels = {};
        if     style == "no"
        then   if field("misc/ST/1", u) == "5" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-ic"].label);      end;
               if field("misc/ST/2", u) == "2" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-injury"].label);  end;
               if field("misc/ST/3", u) == "2" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-death"].label);   end;
               if field("misc/ST/4", u) == "2" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-romance"].label); end;
        elseif style == "ask"
        then   if field("misc/ST/1", u) == "2" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-ic"].label);      end;
               if field("misc/ST/1", u) == "3" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-ic"].label);      end;
               if field("misc/ST/1", u) == "4" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-ic"].label);      end;
               if field("misc/ST/2", u) == "3" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-injury"].label);  end;
               if field("misc/ST/3", u) == "3" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-death"].label);   end;
               if field("misc/ST/4", u) == "3" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-romance"].label); end;
        elseif style == "yes"
        then   if field("misc/ST/1", u) == "1" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-ic"].label);      end;
               if field("misc/ST/2", u) == "1" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-injury"].label);  end;
               if field("misc/ST/3", u) == "1" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-death"].label);   end;
               if field("misc/ST/4", u) == "1" then table.insert(labels, RPTAGS.cache.tagcache["rp:style-romance"].label); end;
        end; -- if style
  
        if labels == {} then return "" else return table.concat(labels, ", ") end;
  
  end; -- func
  
  local function getExperience(u, request)
        local xpField = field("character/XP");
        if not xpField then return ""; end;
        local s = "XP_" .. xpField;
  
        if     request == "long"
        then   return loc(s)
        elseif request == "short"
        then   return loc(s.."_SHORT")
        elseif request == "rookie" and xpField == 1
        then   return loc("XP_1")
        elseif request == "volunteer" and xpField == 3
        then   return loc("XP_3")
        elseif request == "icon"
        then   return loc(s.."_ICON")
        elseif request == "rookie-icon"
        then   return loc("XP_1_ICON");
        elseif request == "volunteer-icon"
        then   return loc("XP_3_ICON");
        else   return "";
        end; -- if request
  end; -- func
  
  local function getClientInfo(u, request)
        if     not u then return "" end;
        local  unitID = getUnitID(u);
        if     not unitID then return "" end;
        local  knownChars = TRP3_API.register.getCharacterList();
        local  Char = knownChars[unitID]
        if     not Char then return "" end;
        if     not Char.client then return "" end;
        local  knownClient  = Const.CLIENT.LOOKUP[Char.client];
        if     not knownClient
        then   return ""
        elseif request == "long"
        then   return loc("CLIENT_" .. knownClient)
        elseif request == "short"
        then   return loc("CLIENT_" .. knownClient .. "_SHORT")
        elseif request == "version" and Char.clientVersion
        then   return Char.clientVersion
        elseif request == "full" and Char.clientVersion
        then   return loc("CLIENT_" .. knownClient) .. " " .. Char.clientVersion
        elseif request == "icon"
        then   return Const.CLIENT.ICON[knownClient] or ""
        elseif request == "TRP" and knownClient == "TRPE" -- special case, rassa frassa
        then   return loc("CLIENT_TRP")
        elseif request == knownClient
        then   return loc("CLIENT_" .. knownClient)
        else   return ""
        end    -- if knownClient
  end; -- function
  
  local function getGuildInfo(u, request)
    local guildStyle = field("misc/ST/6", u, "UNLIM")
    local guildField = misc("guild", u, "MISC");
    local guildName, guildRankName, guildRankIndex = GetGuildInfo(u)
    
    if     guildField ~= ""  then isGuildIC, guildName, guildRankName = true, guildField, ""
    elseif guildStyle == "1" then isGuildIC, guildName, guildRankName = true, guildName,  guildRankName;
    else                          isGuildIC, guildName, guildRankName = false, "", "";
    end; -- if
  
    if     request == "status" and isGuildIC then return loc("IS_IC") 
    elseif request == "status"               then return loc("IS_OOC")
    elseif request == "name"                 then return guildName
    elseif request == "rank"                 then return guildRankName
    end; -- requestif
  end;
  
  -- functions available under RPTAGS.get.text
  
  Text.age        = getAge;
  Text.birthplace = getBP;
  Text.class      = getClass;
  Text.curr       = getCurr;
  Text.eyes       = getEyes;
  Text.firstname  = getFirstName;
  Text.fulltitle  = getFT;
  Text.gender     = getGender;
  Text.glance     = getGlance;
  Text.height     = getHeight;
  Text.home       = getHome;
  Text.ic         = getIcStatus;
  Text.info       = getInfo;
  Text.lastname   = getLastName;
  Text.misc       = getMiscAndTweakOutput;
  Text.name       = getName;
  Text.npc        = getNpcStatus;
  Text.ooc        = getOocStatus;
  Text.race       = getRace;
  Text.sizebuff   = getSizeBuff;
  Text.status     = getStatus;
  Text.title      = getTitle;
  Text.weight     = getWeight;
  Text.years      = getYears;
  Text.pronoun    = getPronoun;
  Text.pronouns   = getAllPronouns;
  Text.relation   = getRelation;
  Text.knownName  = getKnownName;
  
  -- new ---------------------------------------------------
  Text.note       = scanNote;
  Text.relwho     = getRelWho;
  Text.server     = getServerInfo;
  Text.style      = getRoleplayingStyle;
  Text.xp         = getExperience;
  Text.client     = getClientInfo;
  Text.guild      = getGuildInfo;

  return "get-text";
end); -- startup function
