-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions --  You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- 
local RPTAGS = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function

  local Utils     = RPTAGS.utils;
  local floc      = Utils.locale.floc;
  local loc       = Utils.locale.loc;
  local fmt       = string.format;
  local addon     = RPTAGS.addOnName;
  
  local MIN       = 10;
  local TALL      = 600;
  local WIDE      = 800;
  local NEGTALL   = -1 * TALL;
  local NEGWIDE   = -1 * WIDE;
  local HALFTALL  = TALL / 2;
  local MAXZOOM   = 10;
  local DBLWIDE   = 2 * WIDE;
  
  
  if RPTAGS.mode == "elvui" or RPTAGS.mode == "parallel"  then 
    
    local E, _, _, P, _, _  = unpack(ElvUI); --Import: Engine, Locales, PrivateDB, ProfileDB, GlobalDB
    local RPT       = E:NewModule('rp:tags', 'AceHook-3.0');
    local EP        = LibStub("LibElvUIPlugin-1.0")
    -- local UF        = E:GetModule('UnitFrames');
    RPT.version     = loc("APP_VERSION");
    RPT.versionMinE = 10.49
    RPT.title       = loc("APP_COLOR_NAME");
    P['RPT']        = { ['enable'] = false, }
    
    local function tweakFrameSizeLimits()
      
          -- player unitframe 
          E.Options.args.unitframe.args.player.args.generalGroup.args.height.max =     TALL;
          E.Options.args.unitframe.args.player.args.generalGroup.args.width.max  =  DBLWIDE;
          E.Options.args.unitframe.args.player.args.generalGroup.args.width.min  =      MIN;
          
          E.Options.args.unitframe.args.player.args.infoPanel.args.height.max    = HALFTALL;
    
          E.Options.args.unitframe.args.player.args.name.args.xOffset.min        =  NEGWIDE;
          E.Options.args.unitframe.args.player.args.name.args.xOffset.max        =     WIDE;
          E.Options.args.unitframe.args.player.args.name.args.yOffset.min        =  NEGTALL;
          E.Options.args.unitframe.args.player.args.name.args.yOffset.max        =     TALL;
    
          E.Options.args.unitframe.args.player.args.health.args.xOffset.min            =  NEGWIDE;
          E.Options.args.unitframe.args.player.args.health.args.xOffset.max            =     WIDE;
          E.Options.args.unitframe.args.player.args.health.args.yOffset.min            =  NEGTALL;
          E.Options.args.unitframe.args.player.args.health.args.yOffset.max            =     TALL;
    
          E.Options.args.unitframe.args.player.args.power.args.height.max              = HALFTALL;
          E.Options.args.unitframe.args.player.args.power.args.detachedWidth.max       =  DBLWIDE;
          E.Options.args.unitframe.args.player.args.power.args.detachedWidth.min       =      MIN;
    
          E.Options.args.unitframe.args.player.args.power.args.xOffset.min             =  NEGWIDE;
          E.Options.args.unitframe.args.player.args.power.args.xOffset.max             =     WIDE;
          E.Options.args.unitframe.args.player.args.power.args.yOffset.min             =  NEGTALL;
          E.Options.args.unitframe.args.player.args.power.args.yOffset.max             =     TALL;
    
          E.Options.args.unitframe.args.player.args.portrait.args.camDistanceScale.max =  MAXZOOM;
          E.Options.args.unitframe.args.player.args.portrait.args.width.max            =     WIDE;
    
          -- target unitframe
          E.Options.args.unitframe.args.target.args.generalGroup.args.height.max       =     TALL;
          E.Options.args.unitframe.args.target.args.generalGroup.args.width.max        =  DBLWIDE;
          E.Options.args.unitframe.args.target.args.generalGroup.args.width.min        =      MIN;
    
          E.Options.args.unitframe.args.target.args.infoPanel.args.height.max          = HALFTALL;
    
          E.Options.args.unitframe.args.target.args.name.args.xOffset.min              =  NEGWIDE;
          E.Options.args.unitframe.args.target.args.name.args.xOffset.max              =     WIDE;
          E.Options.args.unitframe.args.target.args.name.args.yOffset.min              =  NEGTALL;
          E.Options.args.unitframe.args.target.args.name.args.yOffset.max              =     TALL;
    
          E.Options.args.unitframe.args.target.args.health.args.xOffset.min            =  NEGWIDE;
          E.Options.args.unitframe.args.target.args.health.args.xOffset.max            =     WIDE;
          E.Options.args.unitframe.args.target.args.health.args.yOffset.min            =  NEGTALL;
          E.Options.args.unitframe.args.target.args.health.args.yOffset.max            =     TALL;
    
          E.Options.args.unitframe.args.target.args.power.args.height.max              = HALFTALL;
          E.Options.args.unitframe.args.target.args.power.args.detachedWidth.max       =  DBLWIDE;
          E.Options.args.unitframe.args.target.args.power.args.detachedWidth.min       =      MIN;
    
          E.Options.args.unitframe.args.target.args.power.args.xOffset.min             =  NEGWIDE;
          E.Options.args.unitframe.args.target.args.power.args.xOffset.max             =     WIDE;
          E.Options.args.unitframe.args.target.args.power.args.yOffset.min             =  NEGTALL;
          E.Options.args.unitframe.args.target.args.power.args.yOffset.max             =     TALL;
    
          E.Options.args.unitframe.args.target.args.portrait.args.camDistanceScale.max =  MAXZOOM;
          E.Options.args.unitframe.args.target.args.portrait.args.width.max            =     WIDE;
    end; 
    function RPT:InsertOptions()
    
        if E.db.RPT.size_enable then tweakFrameSizeLimits(); end;
    
        E.Options.args.RPT = {
                    order = 100,
                    type = 'group',
                    name = RPT.title,
                    args = {
                            header = {
                                    order = 100,
                                    type = "header",
                                    name = fmt(floc("APP_NAME"),
                                                   loc("APP_COLOR_NAME"),
                                                   loc("APP_VERSION"),
                                                   loc("APP_CREATOR"));
                            },                
    
                            intro = {
                                    order = 101,
                                    type = "description",
                                    name = loc("CONFIG_TRP3"),
                                    },
    
                            spacer = {
                                    order = 102,
                                    type = "description",
                                    name = "",
                                    },
    
                            size_enable = {
                                    order = 201,
                                    type = "toggle",
                                    name = loc("CONFIG_TWEAK_ELVUI_LIMITS"),
                                    get = function() return E.db.RPT.size_enable end,
                                    set = function(_, value) 
                                                   E.db.RPT.size_enable = value; 
                                                   E:StaticPopup_Show("GLOBAL_RL"); 
                                          end
                                    },
    
                            morespace = {
                                    order = 202,
                                    type = "description",
                                    name = "",
                                    },
    
                    }
            }        
    
        if not RPT.initialized or not E.private.unitframe.enable then return end
    
    end
    
    function RPT:Initialize() EP:RegisterPlugin(addon, RPT.InsertOptions) end;
    
    E:RegisterModule(RPT:GetName())

  end;
  
  return "mod-elvui";

end); -- startup function  
