-- ------------------------------------------------------------------------------
-- RP Tags 
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.

--Notices:

--You do not have to comply with the license for elements of the material
--in the public domain or where your use is permitted by an applicable
--exception or limitation.

--No warranties are given. The license may not give you all of the
--permissions necessary for your intended use. For example, other rights
--such as publicity, privacy, or moral rights may limit how you use the material.
-- ------------------------------------------------------------------------------

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function
  local Utils      = RPTAGS.utils;
  local TRP3_API   = TRP3_API;
  
  local fmt        = string.format;
  
  local createPage   = Utils.config.setup;
  local reference    = Utils.config.reference;
  local loc          = Utils.locale.loc;
  local floc         = Utils.locale.floc;
  local Register     = Utils.import.trp3.register;
  local hi           = Utils.text.hilite;
  local refreshFrame = Utils.tags.refreshFrame;
  local refreshAll   = Utils.tags.refreshAll;
  local setConfig    = Utils.config.set;
  local Config       = Utils.config.get;
  local handler      = Utils.config.handler;
  local Keys         = RPTAGS.const.CONFIG.KEYS;
  local Callbacks    = RPTAGS.const.CONFIG.CALLBACKS;
  local getUnitID    = Utils.import.trp3.getUnitID; -- can't use get because trp3 loads before get
  local unitIDToInfo = Utils.import.trp3.unitIDToInfo;
  local UnitInRaid   = UnitInRaid;
  local UnitInParty  = UnitInParty;
  
  local EVENT_LOADED   = TRP3_API.events.WORKFLOW_ON_LOADED;
  local EVENT_NEWDATA  = TRP3_API.events.REGISTER_DATA_UPDATED;
  
  local configPages, s = createPage(); -- create our config pages
  -- now we register the structure we just defined
  if s 
  then for _, page in ipairs(configPages)
       do Register.config.page(page); 
       end; -- for
  end; -- if successful

  for  setting, value in pairs(RP_TagsDB)
  do   local key = Keys[setting];
       if key then setConfig(key, value) end;
  end; -- for
  
  TRP3_API.configuration.refreshPage("main_config_rptags_eee_rpuf_panels");

  Register.addon( {
    ["name"]        = fmt(floc("APP_NAME"), loc("APP_SHORT_NAME"), loc("APP_VERSION_MODE")),
    ["description"] = loc("APP_DESC"),
    ["version"]     = loc("APP_VERSION"),
    ["id"]          = loc("APP_ID"),
    ["minVersion"]  = 3,
    ["autoEnable"]  = true,
    ["onStart"]     = function() 
                        Register.callback(EVENT_NEWDATA, 
                          function(unitID)
                            if not unitID then return end
                            -- we're going to listen for trp3 update event and make our unitframes update when it happens
                            --
                            
                            local unit_name, _ = unitIDToInfo(unitID);

                            if getUnitID("target")       == unitID or 
                               getUnitID("player")       == unitID or 
                               getUnitID("focus")        == unitID or 
                               getUnitID("targettarget") == unitID or
                               UnitInRaid(unitID)                  or 
                               UnitInRaid(unit_name)               or
                               UnitInParty(unitID)                 or 
                               UnitInParty(unit_name)  
                            then refreshAll() 
                            end; -- if 

--                             if getUnitID("target")       == unitID then refreshFrame("target")        end;
--                             if getUnitID("player")       == unitID then refreshFrame("player")        end;
--                             if getUnitID("focus")        == unitID then refreshFrame("focus")         end;
--                             if getUnitID("targettarget") == unitID then refreshFrame("targettarget")  end;
--                             if UnitInRaid(unitID)                  then refreshFrame("raid")          end;
--                             if UnitInRaid(unit_name)               then refreshFrame("raid")          end;
--                             if UnitInParty(unitID)                 then refreshFrame("party")         end;
--                             if UnitInParty(unit_name)              then refreshFrame("party")         end; -- ##
                          end);  -- end of our callback handler for new data
                      end -- of the onStart function
     }); -- closes the addon registering table and the function params

  return "mod-trp3";
end); -- startup function
