-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- All-Tags

local RPTAGS      = RPTAGS;
table.insert(RPTAGS.startup, function() -- startup function

  local prepare       = RPTAGS.utils.tags.prepare;

  local RP = RPTAGS.const.TAG;
  local pronounsList = RPTAGS.utils.locale.loc("PRONOUN_TAGS");

  -- core get functions -------------------------------------------------------------
  local field       = RPTAGS.get.core.field;
  local unitID      = RPTAGS.get.core.unitID;

  -- Not really intended for end user use
  -- prepare(RP.."statusnum", function(u, w) return field("character/RP", u); end);
  -- prepare(RP.."unit",      function(u, w) return u;                        end);
  -- prepare(RP.."unitid",    function(u, w) return unitID(u);                end);

  -- --------------------------------------------------------------------------------
  local ageColor              = RPTAGS.get.color.age;
  local ageText               = RPTAGS.get.text.age;
  local birthplaceText        = RPTAGS.get.text.birthplace;
  local classText             = RPTAGS.get.text.class;
  local clientText            = RPTAGS.get.text.client;
  local currText              = RPTAGS.get.text.curr;
  local eyeColor              = RPTAGS.get.color.eye;
  local eyesText              = RPTAGS.get.text.eyes;
  local firstnameText         = RPTAGS.get.text.firstname;
  local fulltitleText         = RPTAGS.get.text.fulltitle;
  local friendColor           = RPTAGS.get.color.friend;
  local guildText             = RPTAGS.get.text.guild;
  local guildColor            = RPTAGS.get.color.guild;
  local partyColor            = RPTAGS.get.color.party;
  local genderColor           = RPTAGS.get.color.gender;
  local genderText            = RPTAGS.get.text.gender;
  local genderIcon            = RPTAGS.get.icon.gender;
  local glance                = RPTAGS.get.text.glance;
  local glancesIcon           = RPTAGS.get.icon.glances;
  local heightColor           = RPTAGS.get.color.height;
  local heightText            = RPTAGS.get.text.height;
  local hiliteColor           = RPTAGS.get.color.hilite;
  local homeText              = RPTAGS.get.text.home;
  local icText                = RPTAGS.get.text.ic;
  local infoText              = RPTAGS.get.text.info;
  local knownNameText         = RPTAGS.get.text.knownName;
  local lastnameText          = RPTAGS.get.text.lastname;
  local miscText              = RPTAGS.get.text.misc; -- better than Get.core.misc because it does tweaks as well
  local nameColor             = RPTAGS.get.color.name;
  local nameText              = RPTAGS.get.text.name;
  local npcText               = RPTAGS.get.text.npc;
  local oocText               = RPTAGS.get.text.ooc;
  local profileSizeColor      = RPTAGS.get.color.profileSize;
  local profileSizeCore       = RPTAGS.get.core.profileSize;
  local pronounText           = RPTAGS.get.text.pronoun;
  local pronounsText          = RPTAGS.get.text.pronouns;
  local pvpIcon               = RPTAGS.get.icon.pvp;
  local raceText              = RPTAGS.get.text.race;
  local relWhoText            = RPTAGS.get.text.relwho;
  local relationColor         = RPTAGS.get.color.relation;
  local relationText          = RPTAGS.get.text.relation;
  local relationIcon          = RPTAGS.get.icon.relation;
  local scanNote              = RPTAGS.get.text.note;
  local serverText            = RPTAGS.get.text.server;
  local sizebuffText          = RPTAGS.get.text.sizebuff;
  local statusColor           = RPTAGS.get.color.status;
  local statusText            = RPTAGS.get.text.status;
  local statusIcon            = RPTAGS.get.icon.status;
  local styleText             = RPTAGS.get.text.style;
  local titleText             = RPTAGS.get.text.title;
  local unitIcon              = RPTAGS.get.icon.unit;
  local weightColor           = RPTAGS.get.color.weight;
  local weightText            = RPTAGS.get.text.weight;
  local xpText                = RPTAGS.get.text.xp;
  local yearsText             = RPTAGS.get.text.years;

    -- tag definitions -------------------------------------------------------------------------
  -- prepare(RP.."client-icon",     function(u)    return clientText(u, "icon");           end); commented out because i don't think enough of them have icons.
  prepare("br",                     function()     return "\n";                            end);
  prepare("nocolor",                function()     return "|r";                            end);
  prepare("p",                      function()     return "\n\n";                          end);
  prepare(RP.."age",                function(u, w) return ageText(u, w);                   end);
  prepare(RP.."agecolor",           function(u)    return ageColor(u);                     end);
  prepare(RP.."birthplace",         function(u, w) return birthplaceText(u, w);            end);
  prepare(RP.."body",               function(u, w) return weightText(u, true);             end);
  prepare(RP.."class",              function(u, w) return classText(u, w);                 end);
  prepare(RP.."client",             function(u)    return clientText(u, "long");           end);
  prepare(RP.."client-full",        function(u)    return clientText(u, "full");           end);
  prepare(RP.."client-short",       function(u)    return clientText(u, "short");          end);
  prepare(RP.."client-version",     function(u)    return clientText(u, "version");        end);
  prepare(RP.."color",              function(u)    return nameColor(u);                    end);
  prepare(RP.."curr",               function(u, w) return currText(u, w);                  end);
  prepare(RP.."extended",           function(u)    return clientText(u, "TRPE");           end);
  prepare(RP.."eyecolor",           function(u)    return eyeColor(u);                     end);
  prepare(RP.."eyes",               function(u, w) return eyesText(u, w);                  end);
  prepare(RP.."firstname",          function(u, w) return firstnameText(u, w);             end);
  prepare(RP.."friendcolor",        function(u)    return friendColor(u)                   end, "FRIENDLIST_UPDATE BN_FRIEND_INFO_CHANGED IGNORELIST_UPDATE" );
  prepare(RP.."fulltitle",          function(u, w) return fulltitleText(u, w);             end);
  prepare(RP.."gender",             function(u)    return genderText(u);                   end);
  prepare(RP.."gendercolor",        function(u)    return genderColor(u);                  end);
  prepare(RP.."glance",             function(u, w) return glance(u, 0,1,0,0, not w);       end); -- number, title, text, icon, nowrap
  prepare(RP.."glance-1",           function(u, w) return glance(u, 1,1,0,0, not w);       end);
  prepare(RP.."glance-1-full",      function(u, w) return glance(u, 1,1,1,1, not w);       end);
  prepare(RP.."glance-1-icon",      function(u)    return glance(u, 1,0,0,1,  true);       end);
  prepare(RP.."glance-1-text",      function(u, w) return glance(u, 1,0,1,0, not w);       end);
  prepare(RP.."glance-2",           function(u, w) return glance(u, 2,1,0,0, not w);       end);
  prepare(RP.."glance-2-full",      function(u, w) return glance(u, 2,1,1,1, not w);       end);
  prepare(RP.."glance-2-icon",      function(u)    return glance(u, 2,0,0,1,  true);       end);
  prepare(RP.."glance-2-text",      function(u, w) return glance(u, 2,0,1,0, not w);       end);
  prepare(RP.."glance-3",           function(u, w) return glance(u, 3,1,0,0, not w);       end);
  prepare(RP.."glance-3-full",      function(u, w) return glance(u, 3,1,1,1, not w);       end);
  prepare(RP.."glance-3-icon",      function(u)    return glance(u, 3,0,0,1,  true);       end);
  prepare(RP.."glance-3-text",      function(u, w) return glance(u, 3,0,1,0, not w);       end);
  prepare(RP.."glance-4",           function(u, w) return glance(u, 4,1,0,0, not w);       end);
  prepare(RP.."glance-4-full",      function(u, w) return glance(u, 4,1,1,1, not w);       end);
  prepare(RP.."glance-4-icon",      function(u)    return glance(u, 4,0,0,1,  true);       end);
  prepare(RP.."glance-4-text",      function(u, w) return glance(u, 4,0,1,0, not w);       end);
  prepare(RP.."glance-5",           function(u, w) return glance(u, 5,1,0,0, not w);       end);
  prepare(RP.."glance-5-full",      function(u, w) return glance(u, 5,1,1,1, not w);       end);
  prepare(RP.."glance-5-icon",      function(u)    return glance(u, 5,0,0,1,  true);       end); 
  prepare(RP.."glance-5-text",      function(u, w) return glance(u, 5,0,1,0, not w);       end);
  prepare(RP.."glance-all",         function(u, w) return glance(u, 0,1,0,1, not w);       end);
  prepare(RP.."glance-full",        function(u, w) return glance(u, 0,1,1,1, not w);       end); -- ##
  prepare(RP.."guildcolor",         function(u)    return guildColor(u)                    end, "PLAYER_GUILD_UPDATE GUILD_ROSTER_UPDATE");
  prepare(RP.."height",             function(u)    return heightText(u);                   end);
  prepare(RP.."heightcolor",        function(u)    return heightColor(u);                  end);
  prepare(RP.."hilite-1",           function()     return hiliteColor(1);                  end);
  prepare(RP.."hilite-2",           function()     return hiliteColor(2);                  end);
  prepare(RP.."hilite-3",           function()     return hiliteColor(3);                  end);
  prepare(RP.."home",               function(u, w) return homeText(u, w);                  end);
  prepare(RP.."ic",                 function(u)    return icText(u);                       end);
  prepare(RP.."info",               function(u, w) return infoText(u, w);                  end);
  prepare(RP.."lastname",           function(u, w) return lastnameText(u, w);              end);
  prepare(RP.."mrp",                function(u)    return clientText(u, "MRP");            end);
  prepare(RP.."name",               function(u, w) return nameText(u, w);                  end);
  prepare(RP.."name-known ",        function(u)    return knownNameText(u);                end);
  prepare(RP.."note-1",             function(u)    return scanNote(u, 1);                  end);
  prepare(RP.."note-1-icon",        function(u)    return scanNote(u, false, 1, "ICON");   end);
  prepare(RP.."note-2",             function(u)    return scanNote(u, 2);                  end);
  prepare(RP.."note-2-icon",        function(u)    return scanNote(u, false, 2, "ICON");   end);
  prepare(RP.."note-3",             function(u)    return scanNote(u, 3);                  end);
  prepare(RP.."note-3-icon",        function(u)    return scanNote(u, false, 3, "ICON");   end);
  prepare(RP.."npc",                function(u)    return npcText(u);                      end);
  prepare(RP.."ooc",                function(u)    return oocText(u);                      end);
  prepare(RP.."partycolor",         function(u)    return partyColor(u)                    end, "UNIT_OTHER_PARTY_CHANGED GROUP_JOINED GROUP_LEFT");
  prepare(RP.."profilesize",        function(u, w) return profileSizeCore(u, w);           end);
  prepare(RP.."profilesizecolor",   function(u)    return profileSizeColor(u);             end);
  prepare(RP.."race",               function(u, w) return raceText(u, w);                  end);
  prepare(RP.."relation",           function(u)    return relationText(u);                 end);
  prepare(RP.."relation-who",       function(u, w) return relWhoText(u, w)                 end);
  prepare(RP.."relationcolor",      function(u)    return relationColor(u)                 end);
  prepare(RP.."sizebuff",           function(u)    return sizebuffText(u);                 end, "UNIT_AURA");
  prepare(RP.."status",             function(u)    return statusText(u);                   end);
  prepare(RP.."statuscolor",        function(u)    return statusColor(u);                  end);
  prepare(RP.."title",              function(u, w) return titleText(u, w);                 end);
  prepare(RP.."trp",                function(u)    return clientText(u, "TRP");            end);
  prepare(RP.."weight",             function(u)    return weightText(u);                   end);
  prepare(RP.."weightcolor",        function(u)    return weightColor(u);                  end);
  prepare(RP.."xrp",                function(u)    return clientText(u, "XRP");            end);
  prepare(RP.."years",              function(u, w) return yearsText(u, w);                 end);
  prepare(RP.."years-ago",          function(u, w) return yearsText(u, w, "ago");          end);
  prepare(RP.."years-old",          function(u, w) return yearsText(u, w, "old");          end);
  prepare(RP.."gendericon",         function(u)    return genderIcon(u);                   end);
  prepare(RP.."glance-icons",       function(u)    return glancesIcon(u);                  end);
  prepare(RP.."pvpicon",            function(u)    return pvpIcon(u);                      end, "PVP_ROLE_UPDATE" );
  prepare(RP.."relationicon",       function(u)    return relationIcon(u);                 end);
  prepare(RP.."statusicon",         function(u)    return statusIcon(u);                   end);
  prepare(RP.."icon",               function(u)    return unitIcon(u);                     end);
  prepare(RP.."alignment",          function(u, w) return miscText("alignment",   u, w);   end);
  prepare(RP.."birthday",           function(u, w) return miscText("birthday",    u, w);   end);
  prepare(RP.."faceclaim",          function(u, w) return miscText("faceclaim",   u, w);   end);
  prepare(RP.."family",             function(u, w) return miscText("family",      u, w);   end);
  prepare(RP.."hair",               function(u, w) return miscText("hair",        u, w);   end);
  prepare(RP.."house",              function(u, w) return miscText("house",       u, w);   end);
  prepare(RP.."markings",           function(u, w) return miscText("markings",    u, w);   end);
  prepare(RP.."motto",              function(u, w) return miscText("motto",       u, w);   end);
  prepare(RP.."nick",               function(u, w) return miscText("nickname",    u, w);   end);
  prepare(RP.."nick-quoted",        function(u, w) return miscText("nick-quoted", u, w);   end);
  prepare(RP.."physiognomy",        function(u, w) return miscText("physiognomy", u, w);   end);
  prepare(RP.."piercings",          function(u, w) return miscText("piercings",   u, w);   end);
  prepare(RP.."pronouns",           function(u, w) return miscText("pronouns",    u, w);   end);
  prepare(RP.."religion",           function(u, w) return miscText("religion",    u, w);   end);
  prepare(RP.."rstatus",            function(u, w) return miscText("rstatus",     u, w);   end);
  prepare(RP.."sexuality",          function(u, w) return miscText("sexual",      u, w);   end);
  prepare(RP.."tattoos",            function(u, w) return miscText("tattoos",     u, w);   end);
  prepare(RP.."tribe",              function(u, w) return miscText("tribe",       u, w);   end);
  prepare(RP.."voiceclaim",         function(u, w) return miscText("voiceclaim",  u, w);   end);
  prepare(RP.."pronouns",           function(u, w) return pronounsText(u, w)               end);
  prepare(RP.."style-battle",       function(u)    return styleText(u, "battle")           end);
  prepare(RP.."style-battle-long",  function(u)    return styleText(u, "battle",  "LONG")  end);
  prepare(RP.."style-battle-icon",  function(u)    return styleText(u, "battle",  "ICON")  end);
  prepare(RP.."style-death",        function(u)    return styleText(u, "death")            end);
  prepare(RP.."style-death-long",   function(u)    return styleText(u, "death",   "LONG")  end);
  prepare(RP.."style-death-icon",   function(u)    return styleText(u, "death",   "ICON")  end);
  prepare(RP.."style-guild",        function(u)    return styleText(u, "guild")            end);
  prepare(RP.."style-guild-long",   function(u)    return styleText(u, "guild",   "LONG")  end);
  prepare(RP.."style-guild-icon",   function(u)    return styleText(u, "guild",   "ICON")  end);
  prepare(RP.."style-ic",           function(u)    return styleText(u, "ic")               end);
  prepare(RP.."style-ic-long",      function(u)    return styleText(u, "ic",      "LONG")  end);
  prepare(RP.."style-ic-icon",      function(u)    return styleText(u, "ic",      "ICON")  end);
  prepare(RP.."style-injury",       function(u)    return styleText(u, "injury")           end);
  prepare(RP.."style-injury-long",  function(u)    return styleText(u, "injury",  "LONG")  end);
  prepare(RP.."style-injury-icon",  function(u)    return styleText(u, "injury",  "ICON")  end);
  prepare(RP.."style-romance",      function(u)    return styleText(u, "romance")          end);
  prepare(RP.."style-romance-long", function(u)    return styleText(u, "romance", "LONG")  end);
  prepare(RP.."style-romance-icon", function(u)    return styleText(u, "romance", "ICON")  end);
  prepare(RP.."style-yes",          function(u)    return styleText(u, "yes")              end);
  prepare(RP.."style-no",           function(u)    return styleText(u, "no")               end);
  prepare(RP.."style-ask",          function(u)    return styleText(u, "ask")              end);
  prepare(RP.."style-icons",        function(u)    return styleText(u, "icons")            end);
  prepare(RP.."experience",         function(u)    return xpText(u, "long");               end);
  prepare(RP.."xp",                 function(u)    return xpText(u, "short");              end);
  prepare(RP.."rookie",             function(u)    return xpText(u, "rookie");             end);
  prepare(RP.."volunteer",          function(u)    return xpText(u, "volunteer");          end);
  prepare(RP.."xp-icon",            function(u)    return xpText(u, "icon");               end);
  prepare(RP.."rookie-icon",        function(u)    return xpText(u, "rookie-icon");        end);
  prepare(RP.."volunteer-icon",     function(u)    return xpText(u, "volunteer-icon");     end);
  prepare(RP.."server-abbr",        function(u)    return serverText(u, "abbr"      )      end); -- MG
  prepare(RP.."server-dash",        function(u)    return serverText(u, "dash"      )      end); -- "-"
  prepare(RP.."server-lang",        function(u)    return serverText(u, "lang"      )      end); -- English
  prepare(RP.."server-lang-short",  function(u)    return serverText(u, "lang-short")      end); -- enUS
  prepare(RP.."server-mine",        function(u)    return serverText(u, "mine"      )      end); -- name only if yours
  prepare(RP.."server",             function(u)    return serverText(u, "name"      )      end); -- Moon Guard
  prepare(RP.."server-notmine",     function(u)    return serverText(u, "notmine"   )      end); -- name only if not yours
  prepare(RP.."server-region",      function(u)    return serverText(u, "region"    )      end); -- United States
  prepare(RP.."server-star",        function(u)    return serverText(u, "star"      )      end); -- (*) if not my server
  prepare(RP.."server-subregion",   function(u)    return serverText(u, "subregion" )      end); -- Oceania
  prepare(RP.."server-type",        function(u)    return serverText(u, "type"      )      end); -- Roleplaying
  prepare(RP.."server-type-short",  function(u)    return serverText(u, "type-short")      end); -- RPf
  prepare(RP.."guild",              function(u)    return guildText(u, "name")             end, "GUILD_ROSTER_UPDATE"); -- if guild membership is IC then guild
  prepare(RP.."guild-rank",         function(u)    return guildText(u, "rank")             end, "GUILD_ROSTER_UPDATE"); -- if guild membership is IC then rank
  prepare(RP.."guild-status",       function(u)    return guildText(u, "status")           end, "GUILD_ROSTER_UPDATE"); -- [IC] or [OOC]
-- ##
  for p in string.gmatch(pronounsList, "%a")
  do  prepare(RP..p,                function(u) return pronounText(u, p)                   end);
      prepare(RP..string.upper(p),  function(u) return pronounText(u, string.upper(p))     end);
  end; -- for p

  RPTAGS.utils.tags.registerall();
  return "all-tags";
end); -- startup function
