-- ------------------------------------------------------------------------------
-- RP Tags 
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Config: Utilities for reading and registering configuration values

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function()
  local Const      = RPTAGS.const;
  local Utils      = RPTAGS.utils;
  local loc        = Utils.locale.loc;
  local floc       = Utils.locale.floc;
  local fmt        = string.format;
  
  if not Utils.config then Utils.config = {} end;
  local Config = Utils.config;
  
  local Keys        = Const.CONFIG.KEYS;
  local Default     = Const.CONFIG.DEFAULTS;
  local Page        = Const.CONFIG.PAGES;
  local getValue    = Utils.import.trp3.getValue;
  local setValue    = Utils.import.trp3.setValue;
  local registerHandler = Utils.import.trp3.registerHandler;
  
        -- return the value of a config setting
  local function rptagsConfig(setting)
        if not setting then return nil; end;
         
        local key = Keys[setting] or nil;
  
        if    key then return getValue(key) or Default[setting] or nil
                  else return nil;
        end; -- if;
  end;  
  
  local function rptagsSetConfig(setting, value)
        if not setting then return nil; end;
        if not type(value) == string then return nil; end;
        local key = Keys[setting] or nil;
        if key then return setValue(key, value) or Default[setting] or nil;
               else return nil;
        end; -- if;
  end; -- function
  
  local function rptagsResetConfig(setting)
        local default = Default[setting] or nil;
        if default then rptagsSetConfig(setting, default)
           else return nil;
        end; --if
  end; -- function
  
  local function rptagsGetDefault(setting)
        local default = Default[setting] or nil;
        return default or ""
  end; -- function
  
  local function rptagsResetConfigGroup(settings, page)
        for _, s in ipairs(settings) do rptagsResetConfig(s, p, update, resize) end -- for
        if page then TRP3_API.configuration.refreshPage(page) end;
  end; -- function
        
  local function persistSetting(setting, value)
        if     not value and not setting then return nil 
        elseif not value                 then return RP_TagsDB[setting] or nil; 
        else   RP_TagsDB[setting] = value     return true;
        end;
  end; -- function
        
  local function tagEditBox(setting)
    local data = { setting = setting, current = Config.get(setting), };
    StaticPopup_Show("RPTAGS_TAG_EDIT_BOX", nil, nil, data); 
  end;
  
  local function refreshTRP3ConfigPage(key)
    local page;
    if     Page[key] then TRP3_API.configuration.refreshPage(Page[key])
    else for page, id in pairs(Const.CONFIG.CONFIG_PAGES)
         do TRP3_API.configuration.refreshPage(id)
         end; -- for
    end; -- if Page[key]
  end; 

  local function testTags(s)
    local err = false;
    local good = {};
    local bad = {};
    
    for tag in string.gmatch(s, "%[([^%]]+)%]")
        do if RPTAGS.utils.tags.exists(tag) 
              then table.insert(good, tag)
              else table.insert(bad, tag); err = true;
           end; --if
        end; -- do
    if err then print(fmt(floc("TAG_TEST_FAIL"), table.concat(bad, ", ")))
           else print(loc("FMT_TAG_TEST_PASS"))
           end;
    return err, good, bad; 
  end;
  
  -- Popup dialogs
  
  StaticPopupDialogs["RPTAGS_SAVE_AND_RELOAD"] = {
     showAlert = 1,
     text = loc("RELOAD_UI_WARNING"),
     button1 = loc("RELOAD_UI_WARNING_YES"),
     button2 = loc("RELOAD_UI_WARNING_NO"),
     exclusive = true,
     OnAccept = function() RP_TagsDB.DISABLE_BLIZZARD = not(RP_TagsDB.DISABLE_BLIZZARD); ReloadUI() end,
     timeout = 60,
     whileDead = true,
     hideOnEscape = true,
     preferredIndex = 3,
  }
  
  StaticPopupDialogs["RPTAGS_CHANGE_MODE"] = {
     showAlert = 1,
     OnShow = function(self) self:SetWidth(500) end,
     text = loc("MODE_CHANGE_WARNING_" .. RPTAGS.mode:upper()),
     button1 = loc("MODE_CHANGE_" .. RPTAGS.mode:upper()),
     button2 = loc("MODE_STAY_" .. (RPTAGS.mode == "parallel" and "PARALLEL" or "ELVUI")),
     exclusive = true,
     OnAccept = function() if RP_TagsDB.mode == "parallel" 
                           then RP_TagsDB.mode = "elvui"
                                RPTAGS.utils.config.set("HIDDEN_RPUF_WITH_ELVUI", false)
                           else RP_TagsDB.mode = "parallel"
                                RPTAGS.utils.config.set("HIDDEN_RPUF_WITH_ELVUI", true)
                           end; -- if
                           ReloadUI(); end,
     timeout = 60,
     whileDead = true,
     hideOnEscape = true,
     preferredIndex = 3,
  }
  
  -- Functions available under RPTAGS.utils.config
  --
  Config.tagedit  = tagEditBox;
  Config.get      = rptagsConfig;
  Config.set      = rptagsSetConfig;
  Config.reset    = rptagsResetConfig;
  Config.default  = rptagsGetDefault;
  Config.resetgroup = rptagsResetConfigGroup;
  Config.persist  = persistSetting;
  Config.register = Utils.import.trp3.register.config; 
  Config.testtags = Config.testTags;
  Config.refresh  = refreshTRP3ConfigPage;

  return "utils-config"
end); -- startup function
