-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Config: Utilities for reading and registering configuration values

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function

  local Const      = RPTAGS.const;
  local Utils      = RPTAGS.utils;
  
  if not RPTAGS.cache then RPTAGS.cache = {} end;
  if not Utils.config then Utils.config = {} end;
  if not Utils.help   then Utils.help = {} end;
  local Config = Utils.config;
  
  local ConfigPages = Const.CONFIG.CONFIG_PAGES;
  local currentPage;
  local Pages = Const.CONFIG.PAGES;
  
  local loc         = Utils.locale.loc;
  local hi          = Utils.text.hilite;
  local html        = Utils.text.html;
  local refreshAll  = Utils.tags.refreshAll;
  local Format      = Utils.format;                 -- the subcategory
  local Register    = Utils.import.trp3.register.config;
  
  local Keys        = Const.CONFIG.KEYS;
  local Default     = Const.CONFIG.DEFAULTS;
  local Callback    = Const.CONFIG.CALLBACKS;
  local BIG_PROFILE = Const.BIG_PROFILE;
  local H1          = "TRP3_ConfigH1";
  local CHECK       = "TRP3_ConfigCheck";
  local SELECT      = "TRP3_ConfigDropDown";
  local COLOR       = "TRP3_ConfigColorPicker";
  local BUTTON      = "TRP3_ConfigButton";
  local SLIDER      = "TRP3_ConfigSlider";
  local BOX         = "TRP3_ConfigEditBox";
  local DEPS        = "dependentOnOptions"; 
  local WIDGET      = "TRP3_ConfigurationWidget";
  
        -- load our configuration settings and defaults into trp3
  local function setUpConfig()
    for k, v in pairs(Keys)
    do  Register.key(v, Default[k]);
        Register.handler(v, Callback[k] or refreshAll);
    end -- for
  
  -- these set up the pulldown menus for the config page
    local    weights,     heights, sizebuffs, profilesizes, _;
          _, weights      = Format.kg(73);
          _, heights      = Format.cm(168);
          _, sizebuffs    = Format.sizebuff(8212);
          _, profilesizes = Format.sizewords(7, BIG_PROFILE);
  
  -- used below
    local TRUNC_OR_WRAP = {
        { loc("OPT_LINE_TRUNCATE"),     "TRUNCATE",     },
        { loc("OPT_LINE_WORD_TRUNC"),   "WORD_TRUNC",   },
        { loc("OPT_LINE_WRAP_ONCE"),    "WRAP_ONCE",    },
        { loc("OPT_LINE_WRAP_TWICE"),   "WRAP_TWICE",   },
        { loc("OPT_LINE_WRAP_FOREVER"), "WRAP_FOREVER", },
      };
  
    local RPUF_LAYOUTS = {
        { loc("RPUF_FULL"), "FULL", },
        { loc("RPUF_ABRIDGED"), "ABRIDGED", },
        { loc("RPUF_COMPACT"), "COMPACT", },
        { loc("RPUF_PAPERDOLL"), "PAPERDOLL", },
        { loc("RPUF_THUMBNAIL"), "THUMBNAIL", },
  --      { loc("RPUF_HIDDEN"), "HIDDEN", },
      };
    local RPUF_LAYOUTS_SMALL = {
        { loc("RPUF_ABRIDGED"), "ABRIDGED", },
        { loc("RPUF_COMPACT"), "COMPACT", },
        { loc("RPUF_THUMBNAIL"), "THUMBNAIL", },
  --      { loc("RPUF_HIDDEN"), "HIDDEN", },
      };
  
  RPTAGS.cache.layoutOptions = {
     RPUF_Player       = { choices = RPUF_LAYOUTS,       config = "PLAYERLAYOUT" },
     RPUF_Target       = { choices = RPUF_LAYOUTS,       config = "TARGETLAYOUT" },
     RPUF_Focus        = { choices = RPUF_LAYOUTS_SMALL, config = "FOCUSLAYOUT"  },
     RPUF_TargetTarget = { choices = RPUF_LAYOUTS_SMALL, config = "FOCUSLAYOUT"  },
     };

  -- this works with the above to finish setting up the pulldowns
    local LIST = { -- yeah there are probably easier ways to do this but whatever
      UNITS_HEIGHT = { 
        { heights["CM"],              "CM" },
        { heights["CM_FT_IN"],        "CM_FT_IN" },
        { heights["CM_IN"],           "CM_IN" },
        { heights["FT_IN"],           "FT_IN" },
        { heights["FT_IN_CM"],        "FT_IN_CM" },
        { heights["IN"],              "IN" }, 
        { heights["IN_CM"],           "IN_CM" },
        { heights["HEIGHT_PASSTHRU"], "HEIGHT_PASSTHRU" },
       },
      UNITS_WEIGHT = { 
        { weights["KG"],              "KG" },
        { weights["KG_LB"],           "KG_LB" },
        { weights["KG_ST_LB"],        "KG_ST_LB" },
        { weights["LB_KG"],           "LB_KG",}, 
        { weights["LB_ST"],           "LB_ST",},
        { weights["ST_LB"],           "ST_LB",},
        { weights["ST_LB_KG"],        "ST_LB_KG",},
        { weights["ST_LB_LB_KG"],     "ST_LB_LB_KG" },
        { weights["WEIGHT_PASSTHRU"], "WEIGHT_PASSTHRU" },
       },
      PROFILESIZE_FMT = { 
        { profilesizes["WRD_BRC_NUM"], "WRD_BRC_NUM" },
        { profilesizes["WRD_PAR_NUM"], "WRD_PAR_NUM" },
        { profilesizes["BRC_WRD_NUM"], "BRC_WRD_NUM" },
        { profilesizes["PAR_WRD"],     "PAR_WRD" },
        { profilesizes["WRD"],         "WRD" },
        { profilesizes["BRC_LTR_NUM"], "BRC_LTR_NUM" }, 
        { profilesizes["BRC_LTR"],     "BRC_LTR" }, 
        { profilesizes["LTR"],         "LTR",},
        { profilesizes["BRC_NUM"],     "BRC_NUM",},
        { profilesizes["NUM"],         "NUM",},
       },
      SIZEBUFF_FMT = { 
        { sizebuffs["PCT"],          "PCT" },
        { sizebuffs["PCT_BRC"],      "PCT_BRC" },
        { sizebuffs["PCT_BUFF"],     "PCT_BUFF" },
        { sizebuffs["BUFF_PCT"],     "BUFF_PCT" },
        { sizebuffs["BUFF"],         "BUFF" },
        { sizebuffs["BUFF_BRC"],     "BUFF_BRC" },
        { sizebuffs["BUFF_PCT_BRC"], "BUFF_PCT_BRC" },
      },
      GLANCE_COLON = { 
        { loc("CONFIG_GLANCE_COLON_COLON"),   ": "  , },
        { loc("CONFIG_GLANCE_COLON_COMMAS"), ", "  , },
        { loc("CONFIG_GLANCE_COLON_DASHES"),  " - " , },
        { loc("CONFIG_GLANCE_COLON_SLASHES"), " / " , },
        { loc("CONFIG_GLANCE_COLON_SPACES"),  " "   , }, 
      },
      GLANCE_DELIM = { 
        { loc("CONFIG_GLANCE_DELIM_COMMAS"),     ", ",  },
        { loc("CONFIG_GLANCE_DELIM_NEWLINES"),   "\n",  },
        { loc("CONFIG_GLANCE_DELIM_SEMICOLONS"), "; ",  },
        { loc("CONFIG_GLANCE_DELIM_SLASHES"),    " / ", },
        { loc("CONFIG_GLANCE_DELIM_SPACES"),     ", ",  },
      },
      REF_TOPIC   = { }, -- filled in later
      NAME_WRAP   = TRUNC_OR_WRAP,
      TITLE_WRAP  = TRUNC_OR_WRAP,
      GLANCE_WRAP = TRUNC_OR_WRAP,
      GLANCETITLE_WRAP = TRUNC_OR_WRAP,
      MISC_WRAP   = TRUNC_OR_WRAP,
      CURR_WRAP   = TRUNC_OR_WRAP,
      PLAYERLAYOUT = RPUF_LAYOUTS,
      TARGETLAYOUT = RPUF_LAYOUTS,
      FOCUSLAYOUT  = RPUF_LAYOUTS_SMALL,
      PARTYLAYOUT  = RPUF_LAYOUTS,
      RAIDLAYOUT   = RPUF_LAYOUTS_SMALL,
      RPUF_WITH_ELVUI = {
        { loc("MODE_PARALLEL"), true,       },
        { loc("MODE_ELVUI"),    false,      },
      },
      RPUF_BACKDROP = {
        { loc("BACKDROP_ORIGINAL"), "ORIGINAL", },
        { loc("BACKDROP_BLIZZTOOLTIP"), "BLIZZTOOLTIP", },
      },
     }; -- closes the LIST table
  
    -- local PORTRAIT_TYPE = {};
    -- for portraitType, _ in pairs(RPTAGS.const.ANIM)
    -- do  table.insert(PORTRAIT_TYPE, { loc("PORTRAIT_TYPE_" .. portraitType), portraitType} );
    -- end; 
    -- table.sort(PORTRAIT_TYPE, function(a, b) return a[1] < b[1] end);
    -- LIST.PORTRAIT_TYPE = PORTRAIT_TYPE;
        
    local tempList = {};
    local default;
    for topic, _ in pairs(RPTAGS.cache.helptopics) 
      do local  title = loc("HELPTITLE_" .. topic);
         if     title:match("^%- DEFAULT: ")
         then   -- print("|cff00dd00", "Skipping the Default!", "|r");
         elseif title:match("^%- ")
         then   table.insert(LIST.REF_TOPIC, { "|cffdddd00" .. title:gsub("^%- ", "") .. "|r", topic }); 
                -- print("|cffdddd00", "Match: ", "|r", loc("HELPTITLE_" .. topic));
                --          (--------------, { ------------------------------------------, ----- });
         else   table.insert(tempList,       { loc("HELPTITLE_" .. topic)                , topic });
                -- print("|cffdd0000", "NOT a Match: ", "|r", loc("HELPTITLE_" .. topic));
         end;   -- if
    end; -- for
    table.sort(LIST.REF_TOPIC, function(a, b) return a[1] < b[1] end);
    table.sort(tempList,       function(a, b) return a[1] < b[1] end);
    for _, v in ipairs(tempList) do table.insert(LIST.REF_TOPIC, v) end;
    -- table.insert(LIST.REF_TOPIC, 1, default);
  
     -- these helper functions define various types of trp3 help screen widgets
    local function h1(key)
      Pages[key] = currentPage;
      return { 
        inherit     = H1,
        title       = loc("CONFIG_H1_" .. key) }; 
      end; -- h1()
  
    local function check(key, dep)
      Pages[key] = currentPage;
      return { 
        inherit     = CHECK,
        title       = hi(loc("CONFIG_" .. key)),
        widgetName  = WIDGET .. Keys[key],
        configKey   = Keys[key],
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = dep }; 
      end; -- check()
  
    local function list(key, dep)
      Pages[key] = currentPage;
      local help = hi(loc("CONFIG_" .. key .. "_TT"));
      if string.len(help) == 0 then help = nil; end;
      return { 
        inherit     = SELECT,
        title       = hi(loc("CONFIG_" .. key)),
        configKey   = Keys[key],
        widgetName  = WIDGET .. Keys[key],
        listContent = LIST[key], 
        help        = help,
        [DEPS]      = dep }; 
      end; -- list()
  
    local function slider(key, dep, min, max, step)
      Pages[key] = currentPage;
      local help = hi(loc("CONFIG_" .. key .. "_TT"));
      if string.len(help) == 0 then help = nil; end;
      return {
          inherit   = SLIDER,
          title     = hi(loc("CONFIG_" .. key)),
          configKey = Keys[key],
          widgetName = WIDGET .. Keys[key],
          min       = min or 0,
          max       = max or 60,
          step      = step or 5,
          integer   = true,
          help      = help,
          [DEPS]    = dep }; 
       end; -- slider()
  
    local function color(key, dep)
      Pages[key] = currentPage;
      key = "COLOR_" .. key; 
      return { 
        inherit     = COLOR,
        title       = hi(loc("CONFIG_" .. key)),
        configKey   = Keys[key],
        widgetName = WIDGET .. Keys[key],
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = dep }; 
      end; -- color()
  
    local function button(key, text, deps)
      Pages[key] = currentPage;
      local title   = hi(loc("CONFIG_BTN_" .. key));
      text          = hi(loc(text or "TOGGLE"));
      return {
        inherit     = BUTTON,
        title       = title,
        configKey   = Keys[key],
        help        = hi(loc("CONFIG_BTN_" .. key .. "_TT")),
          widgetName = WIDGET .. Keys[key],
        [DEPS]      = deps,
        text        = text,
        callback    = Callback[key] or function() print("Beep!") end,
        };
      end; -- button()
  
    local function tagedit(key, label, deps)
      Pages[key] = currentPage;
      return {
        inherit     = BUTTON,
        title       = hi(loc("CONFIG_" .. key)),
        text        = hi(loc(label or "EDIT_TAGS")),
        configKey   = Keys[key],
        widgetName  = WIDGET .. Keys[key],
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = deps,
        callback    = function() Utils.rpuf.tagEdit(key); end,
        };
    end; -- tagedit()
  
    local function link(key, text, instr)
      Pages[key] = currentPage;
      local title   = hi(loc("CONFIG_BTN_" .. key));
      local url     = loc("CONFIG_BTN_" .. key .. "_URL");
      text          = hi(loc(text  or "GET_LINK"));
      instr         = hi(loc(instr or "GET_LINK_INSTRUCTIONS"));
  
      return { 
        inherit     = BUTTON,
        title       = title,
        configKey   = Keys[key],
        widgetName  = WIDGET .. (Keys[key] or "rptags_" .. key),
        help        = hi(loc("CONFIG_BTN_" .. key .. "_TT")),
        [DEPS]      = nil,
        text        = text,
        callback    = Callback[key] or function() Format.popup:OpenURL(url, title, instr); end -- end of callback function                      
        }; 
      end; -- link()
  
    local toggle_blizzard;
  
    if RP_TagsDB.DISABLE_BLIZZARD
       then toggle_blizzard = button("ENABLE_BLIZZARD")
       else toggle_blizzard = button("DISABLE_BLIZZARD")
       end;
  
    local function textbox(key, dep)
      Pages[key] = currentPage;
     return {
        inherit   = BOX,
        title     = hi(loc("CONFIG_" .. key)),
          widgetName = WIDGET .. Keys[key],
        configKey = Keys[key],
        help      = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]    = dep };
      end; -- textbox
  
    local CONF_STRUCT = {}
      -- this creates the structure of the configuration page, which we will pass to trp3
    currentPage = ConfigPages.MAIN;
    table.insert(CONF_STRUCT, 
      { -- [1]
      id       = currentPage,
      menuText = loc("APP_SHORT_COLOR_NAME"),
      pageText = loc("APP_COLOR_NAME") .. " " .. loc("CONFIG_PAGE_MAIN"), 
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity         
        h1("DISPLAY"), check("LOGIN_MESSAGE" ), -- second column is for dependencies
                       check("CHANGES_MESSAGE", { Keys["LOGIN_MESSAGE"] } ), 
                    -- check("IC_ONLY"),   -- removed
                       check("LINEBREAKS"),
        h1("PARSE"),   check("PARSE_HW"),
                       check("PARSE_AGE"),
                       check("PARSE_GENDER"),
                       check("ADULT_GENDERS", { Keys["PARSE_GENDER"] }),
                       textbox("NOTE_1_STRING"),
                       textbox("NOTE_2_STRING"),
                       textbox("NOTE_3_STRING"),
        --  h1("LINKS"), link("CURSE"), 
        --               link("DL"), 
        --               link("WIKI"), 
        --               link("DISC"), 
        --               link("TWTR"),
        --  h1("DEMO"),  link("DEMO",   "COPY_UI",     "COPY_UI_INSTRUCTIONS"    ), 
        --               link("MANUAL", "COPY_MANUAL", "COPY_MANUAL_INSTRUCTIONS"), 
        }, -- elements
      } -- end 
    );

    currentPage = ConfigPages.FORMATTING;
    if RPTAGS.mode == "elvui" or RPTAGS.mode == "parallel" then 
    table.insert(CONF_STRUCT,
      {  -- [2]
      id       = currentPage,
      menuText = loc("APP_COLOR_RP") .. loc("CONFIG_PAGE_FORMATS"), 
      pageText = loc("APP_COLOR_NAME") .. " " .. loc("CONFIG_PAGE_FORMATS"), 
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity         
                     --
          --
        h1("FORMATS"), list("UNITS_HEIGHT"),
                       list("UNITS_WEIGHT"),
                       list("SIZEBUFF_FMT"),
                       list("GLANCE_DELIM"),                     
                       list("GLANCE_COLON"),
                       list("PROFILESIZE_FMT"),
        h1("SIZES"),   slider("NAME_SIZE"   ),
                       slider("CURR_SIZE"   ),
                       slider("GLANCETITLE_SIZE" ),
                       slider("GLANCE_SIZE" ),
                       slider("TITLE_SIZE"  ),
                       slider("MISC_SIZE"   ),
        h1("WRAP"),    check("REAL_ELLIPSES"),
                       list("NAME_WRAP"   ),
                       list("CURR_WRAP"   ),
                       list("GLANCETITLE_WRAP"),
                       list("GLANCE_WRAP" ),
                       list("TITLE_WRAP"  ),
                       list("MISC_WRAP"   ),
        }, -- elements
      } -- end [2]
    );
    else 
    table.insert(CONF_STRUCT,
      {  -- [2]
      id       = currentPage,
      menuText = loc("APP_COLOR_RP") .. loc("CONFIG_PAGE_FORMATS"), 
      pageText = loc("APP_COLOR_NAME") .. " " .. loc("CONFIG_PAGE_FORMATS"), 
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity         
                     --
          --
        h1("FORMATS"), list("UNITS_HEIGHT"),
                       list("UNITS_WEIGHT"),
                       list("SIZEBUFF_FMT"),
                       list("GLANCE_DELIM"),                     
                       list("GLANCE_COLON"),
                       list("PROFILESIZE_FMT"),
        }, -- elements
      } -- end [2]
    );
    end; -- if 
  
    currentPage = ConfigPages.COLORS;

    table.insert(CONF_STRUCT,
      {  -- [3]
      id       = currentPage,
      menuText = loc("APP_COLOR_RP") .. loc("CONFIG_PAGE_COLORS"), 
      pageText = loc("APP_COLOR_NAME") .." ".. loc("CONFIG_PAGE_COLORS"), 
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity         
                       button("RESET_COLORS", "RESET"),
                       color("UNKNOWN"),
      h1("COLORS_STATUS"),
                       color("IC"),
                       color("OOC"),
                       color("NPC"),
      h1("COLORS_GENDER"),
                       color("MALE"),
                       color("FEMALE"),
                       color("NEUTER"),
                       color("THEY", { Keys["PARSE_GENDER"] }),
      h1("COLORS_COMPARISON"),
                       color("LESSTHAN"),
                       color("EQUALISH"),
                       color("GREATERTHAN"),
      h1("COLORS_HILITE"),
                       color("HILITE_1"),
                       color("HILITE_2"),
                       color("HILITE_3"),
        }, -- elements
      } -- end [3]
    );

    currentPage = ConfigPages.REFERENCE;
    table.insert(CONF_STRUCT,
      {  -- [4]
      id      = currentPage,
      menuText = loc("APP_COLOR_RP") .. loc("CONFIG_PAGE_REF"),
      pageText = loc("APP_COLOR_NAME") .. " ".. loc("CONFIG_PAGE_REF"),
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity
                   list("REF_TOPIC"),
        }, -- elements
      } -- end [4]
    );
  
    currentPage = ConfigPages.LAYOUT;
    if RPTAGS.mode == "standalone" then table.insert(CONF_STRUCT, 
      {  -- [5]
      id      = currentPage,
      menuText = loc("CONFIG_PAGE_RPUF_LAYOUT"),
      pageText = loc("CONFIG_PAGE_RPUF_LAYOUT"),
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity
      h1("RPUF_APPEAR"), 
                    list("RPUF_BACKDROP"),
                    color("RPUF"),
                    slider("RPUFALPHA", nil, 0, 100, 5),
      --               list("PORTRAIT_TYPE"),
      h1("LAYOUT"), check("LOCK_FRAMES"),
                    list("PLAYERLAYOUT"),
                    list("TARGETLAYOUT"),
                    list("FOCUSLAYOUT"),
       --           list("PARTYLAYOUT"), -- not yet implemented
       --           list("RAIDLAYOUT"),
                    slider("INFOWIDTH", nil, 100, 400, 10),
                    slider("ICONWIDTH", nil, 10, 50, 1),
                    slider("PORTWIDTH", nil, 25, 200, 5),
                    slider("DETAILHEIGHT", nil, 20, 250, 5),
                    slider("STATUSHEIGHT", nil, 15, 150, 5),
                    slider("GAPSIZE", nil, 0, 20, 1),
      h1("RESET"),  button("RESET_FRAMES", "RESET"),
                    toggle_blizzard,
        },
      } -- end [5]
    );
    elseif RPTAGS.mode == "parallel" then table.insert(CONF_STRUCT, 
      {  -- [5]
      id      = currentPage,
      menuText = loc("CONFIG_PAGE_RPUF_LAYOUT"),
      pageText = loc("CONFIG_PAGE_RPUF_LAYOUT"),
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity
      h1("RPUF_APPEAR"), 
                    list("RPUF_BACKDROP"),
                    color("RPUF"),
                    slider("RPUFALPHA", nil, 0, 100, 5),
    --                 list("PORTRAIT_TYPE"),
      h1("LAYOUT"), check("LOCK_FRAMES"),
                    list("PLAYERLAYOUT"),
                    list("TARGETLAYOUT"),
                    list("FOCUSLAYOUT"),
       --             list("PARTYLAYOUT"),
       --             list("RAIDLAYOUT"),
                    slider("INFOWIDTH", nil, 100, 400, 10),
                    slider("ICONWIDTH", nil, 10, 50, 1),
                    slider("PORTWIDTH", nil, 25, 200, 5),
                    slider("DETAILHEIGHT", nil, 20, 250, 5),
                    slider("STATUSHEIGHT", nil, 15, 150, 5),
                    slider("GAPSIZE", nil, 0, 20, 1),
      h1("RESET"),  button("RESET_FRAMES", "RESET"),
        },
      } -- end [5]
    );
    end; -- if cache.rpuf
  
    currentPage = ConfigPages.PANELS;
    if RPTAGS.mode == "parallel" or RPTAGS.mode == "elvui"
    then table.insert(CONF_STRUCT, 
      {  -- [6]
      id      = currentPage,
      menuText = loc("CONFIG_PAGE_RPUF_PANELS"),
      pageText = loc("CONFIG_PAGE_RPUF_PANELS"),
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity
                    button("RPUF_WITH_ELVUI",    "MODE_CHANGE"),
      h1("PANELS"), tagedit("NAMEPANEL",         "EDIT_PANEL",     { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("NAME_TOOLTIP",      "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("INFOPANEL",         "EDIT_PANEL",     { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("INFO_TOOLTIP",      "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("STATUSPANEL",       "EDIT_PANEL",     { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("STATUS_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("DETAILPANEL",       "EDIT_PANEL",     { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("DETAIL_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("PORTRAIT_TOOLTIP",  "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
      h1("ICONS"),  tagedit("ICON_1",            "EDIT_ICON",      { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_1_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_2",            "EDIT_ICON",      { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_2_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_3",            "EDIT_ICON",      { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_3_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_4",            "EDIT_ICON",      { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_4_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_5",            "EDIT_ICON",      { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_5_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_6",            "EDIT_ICON",      { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
                    tagedit("ICON_6_TOOLTIP",    "EDIT_TOOLTIP",   { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
      h1("RESET"),  button("RESET_TAGS",         "RESET",          { Keys["HIDDEN_RPUF_WITH_ELVUI"] } ),
        },
      } -- end [6]
    );
    else -- standalone
      table.insert(CONF_STRUCT, 
      {  -- [6]
      id      = currentPage,
      menuText = loc("CONFIG_PAGE_RPUF_PANELS"),
      pageText = loc("CONFIG_PAGE_RPUF_PANELS"),
      elements = { -- these are the widgets for each config option
                   -- the h1()s are just set off for source code clarity
      --            check("MODIFY_ELVUI_PANELS"),
      h1("PANELS"), tagedit("NAMEPANEL",            "EDIT_PANEL"  ),
                    tagedit("NAME_TOOLTIP",         "EDIT_TOOLTIP"),
                    tagedit("INFOPANEL",            "EDIT_PANEL"  ),
                    tagedit("INFO_TOOLTIP",         "EDIT_TOOLTIP"),
                    tagedit("STATUSPANEL",          "EDIT_PANEL"  ),
                    tagedit("STATUS_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("DETAILPANEL",          "EDIT_PANEL"  ),
                    tagedit("DETAIL_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("PORTRAIT_TOOLTIP",     "EDIT_TOOLTIP"),
      h1("ICONS"),  tagedit("ICON_1",               "EDIT_ICON"   ),
                    tagedit("ICON_1_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("ICON_2",               "EDIT_ICON"   ),
                    tagedit("ICON_2_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("ICON_3",               "EDIT_ICON"   ),
                    tagedit("ICON_3_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("ICON_4",               "EDIT_ICON"   ),
                    tagedit("ICON_4_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("ICON_5",               "EDIT_ICON"   ),
                    tagedit("ICON_5_TOOLTIP",       "EDIT_TOOLTIP"),
                    tagedit("ICON_6",               "EDIT_ICON"   ),
                    tagedit("ICON_6_TOOLTIP",       "EDIT_TOOLTIP"),
      h1("RESET"),  button("RESET_TAGS",            "RESET"       ),
        },
      } -- end [6]
    );
    end; -- 
  
    RPTAGS.cache.ConfigPage_cached = CONF_STRUCT;
    return CONF_STRUCT, true; -- true because we succeeded in creating the structure
  end; -- function
  
  local reference_menu = CreateFrame('SimpleHTML', "rp_tags_reference_frame", UIParent, "TRP3_DashboardHTMLContent");
  
  local function display_reference(topic)
    local title = loc("HELPTITLE_"  .. topic):gsub("^- ","");
    local body =  loc("HELPTOPIC_"  .. topic):gsub("Comparisons:", "</p><h3>Comparisons</h3><p>"):gsub("Colors:","</p><h3>Colors</h3><p>");
    local html =  Utils.text.html(title, body); 
    reference_menu:SetText(html);
    end;
  
        -- create the reference menu ------------------------------------------------------------------------
  local function create_reference_menu(reference_parent)
    reference_menu:SetParent(reference_parent);
    reference_menu:SetAllPoints();
    reference_menu:SetWidth(400);
  
    reference_menu:SetPoint("TOPLEFT", reference_parent, "BOTTOMLEFT", 0, 0);
    reference_menu:SetPoint("TOPRIGHT", reference_parent, "BOTTOMRIGHT", 0, 0);
  
    reference_menu:SetFontObject("p", GameFontNormal);
    reference_menu:SetFontObject("h1", GameFontNormalHuge3);
    reference_menu:SetFontObject("h2", GameFontNormalHuge);
    reference_menu:SetFontObject("h3", GameFontNormalLarge);
    RPTAGS.cache.reference = reference_menu;
    display_reference(Utils.config.get("REF_TOPIC"));
  end;
  
  -- Functions available under RPTAGS.utils.config
  --
  Config.setup    = setUpConfig;
  Config.reference = create_reference_menu;
  Utils.help.display = display_reference;

  return "utils-setup"

end); -- startup function


