-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Tags: Utilities for creating tags in ElvUI or RPUF
  -- list of tags at the end

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function

  local _G         = _G;
  local ElvUI      = _G["ElvUI"];
  
  local oUF = RPTAGS.oUF;
  
  if not RPTAGS.utils      then RPTAGS.utils      = {}; end;
  if not RPTAGS.utils.tags then RPTAGS.utils.tags = {}; end;
  
  if not RPTAGS.cache          then RPTAGS.cache          = {} end;
  if not RPTAGS.cache.tagcache then RPTAGS.cache.tagcache = {} end;
  
  local function makeCache(tag)
        if not RPTAGS.cache.tagcache[tag] then RPTAGS.cache.tagcache[tag] = {} end;
        return RPTAGS.cache.tagcache[tag];
  end; -- function
  
        -- registers one tag, an event to wait for, and a method to invoke when found --------------------------
  local function registerTag(tag)
        local TagData = RPTAGS.cache.tagcache[tag];
  
        if     TagData.external -- this means it's a tag from outside of our program so we don't need to add it
        then   TagData.registered = true; return false
        else   local events = RPTAGS.const.MAIN_EVENT;
               if TagData.extraEvents then events = events .. " " .. TagData.extraEvents; end;
  
               if   not TagData.registered
               then if   RPTAGS.mode == "elvui"  or RPTAGS.mode == "parallel"
                    then _G["ElvUF"].Tags.Events[tag]  = events;
                         _G["ElvUF"].Tags.Methods[tag] = function(u) return TagData.method(u, true) end;
                    end;
  
                    if   RPTAGS.mode == "standalone" or RPTAGS.mode == "parallel"
                    then oUF.Tags.Events[tag]  = events;
                         oUF.Tags.Methods[tag] = TagData.method; 
                    end; 
  
                    TagData.registered = true;
               end;
        end; -- if not TagData
  end; -- function
  
  local function registerAllTags()
        for tag, _ in pairs(RPTAGS.cache.tagcache)
        do  registerTag(tag);
        end;
  
             -- make sure that we have all the ElvUI tags available to rp:UF
        if   RPTAGS.mode == "parallel"
        then for tag, event in pairs(_G["ElvUF"].Tags.Events)
             do if   not oUF.Tags.Events[tag]
                then     oUF.Tags.Events[tag] = event;
                        oUF.Tags.Methods[tag] = _G["ElvUF"].Tags.Methods[tag]
                end;
             end;
        end;
  end; -- function
  
  local function testIfTagExists(t)
        if t == "curhp" or t == "curpp" or -- oUF has seven "virtual" tags that don't show up in their tag tables
           t == "maxhp" or t == "maxpp" or t == "class" or t == "faction" or t == "race" then return true end;
        return oUF.Tags.Events[t] ~= nil;
  end; -- function
  
  local function prepareTagData(tag, method, extraEvents)
        local  TagData = makeCache(tag);
        if   not TagData["alias-for"] and not TagData["label-for"]
        then TagData.method      = method;
             TagData.extraEvents = extraEvents;
  
             if   TagData.label
             then local LabelData = makeCache(tag .. "-label");
                  LabelData.method = function(u, w) 
                                       local value = method(u, w); 
                                       if    value and value ~= "" 
                                       then  return TagData.label .. ": " .. value; 
                                       end   -- if value
                                     end; -- function
                  LabelData.extraEvents = extraEvents;
             end; --if label
  
             if   TagData.aliases
             then for _, aka in ipairs(TagData.aliases)
                  do  local AliasData = makeCache(aka);
                      AliasData.method = method;
                      AliasData.extraEvents = extraEvents;
                      if   AliasData.label
                      then local  LabelData = makeCache(aka .. "-label");
                           LabelData.method = function(u, w) 
                                                local value = method(u, w); 
                                                if    value and value ~= "" 
                                                then  return AliasData.label .. ": " .. value; 
                                                end   -- if value
                                              end; -- function
                           LabelData.extraEvents = extraEvents;
                      end; -- if alias label
                  end; -- for aka
             end; -- if aliases
        end; -- if not alias-for or label-for
  end;  -- function
  
  local function refreshFrame(frame)
        if RPTAGS.mode == "elvui"      or RPTAGS.mode == "parallel" then RPTAGS.utils.import.elvui.refreshFrame(frame) end;
        if RPTAGS.mode == "standalone" or RPTAGS.mode == "parallel" then RPTAGS.utils.rpuf.frame.refresh(frame);       end;
  end; -- function      
  
  local function refreshAll()
        if RPTAGS.mode == "elvui"      or RPTAGS.mode == "parallel" then RPTAGS.utils.import.elvui.refreshAll(); end;
        if RPTAGS.mode == "standalone" or RPTAGS.mode == "parallel" then RPTAGS.utils.rpuf.frame.refreshall();   end;
  end; -- function
  
  
  RPTAGS.utils.tags.register     = registerTag; --
  RPTAGS.utils.tags.registerall  = registerAllTags;
  RPTAGS.utils.tags.exists       = testIfTagExists;
  RPTAGS.utils.tags.prepare      = prepareTagData;
  RPTAGS.utils.tags.refreshFrame = refreshFrame;
  RPTAGS.utils.tags.refreshAll   = refreshAll;

  return "utils-tags"
end); -- startup function
